// 
// sussen-applet - ActionDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Net;
using System.Security.Cryptography;
using System.Threading;

using Gtk;
using Gdk;

using Sussen.Oval;

namespace Sussen.Applet
{
	public enum ActionDialogType { Scan, Analyze };

	public class ActionDialog
	{
		private Glade.XML ui;
		
		[Glade.Widget] Gtk.Dialog mainDialog;
		[Glade.Widget] Gtk.Image computerImage;
		[Glade.Widget] Gtk.HBox hbox13;
		[Glade.Widget] Gtk.HBox hbox22;
		[Glade.Widget] Gtk.Entry ovalDefinitionsEntry;
		[Glade.Widget] Gtk.HBox systemDataHBox;
		[Glade.Widget] Gtk.Label systemDataLabel;
		[Glade.Widget] Gtk.Entry systemDataFileEntry;
		[Glade.Widget] Gtk.Entry ovalResultsEntry;
		[Glade.Widget] Gtk.CheckButton saveResultsCheckButton;
		[Glade.Widget] Gtk.ProgressBar progressBar;
		[Glade.Widget] Gtk.Label progressLabel;
		[Glade.Widget] Gtk.Button executeButton;

		ComboBox ovalDefinitionsCombo;
		Hashtable ovalDefinitionsHash;
		
		ActionDialogType actionDialogType;
	
		bool scanDone;
		bool userCancelled;
		
		uint scanTimer;
		string xmlDirectory;
		string downloadDirectory;
		string scanResultsFilename;
		ThreadNotify notify;

		string progressString;
		
		public ActionDialog (ActionDialogType dialogType)
		{
			ovalDefinitionsHash = new Hashtable ();
			
			actionDialogType = dialogType;
			scanDone = false;
		
			ui = Glade.XML.FromAssembly ("sussen-applet.glade", "mainDialog", null);
			ui.Autoconnect (this);

			LoadPreference (Preferences.OVAL_XML_DIRECTORY);
			LoadPreference (Preferences.OVAL_DOWNLOAD_DIRECTORY);
			
			ovalResultsEntry.Sensitive = false;
			saveResultsCheckButton.Toggled += new EventHandler (OnSaveResultsToggled);
			
			string xmldir = (string) Preferences.Get (Preferences.OVAL_XML_DIRECTORY);
			
			ovalDefinitionsCombo = ComboBox.NewText ();			
			DirectoryInfo dir = new DirectoryInfo (xmldir);
			if (dir.Exists)
				foreach (FileInfo f in dir.GetFiles("*.xml")) {
					ovalDefinitionsCombo.AppendText (f.Name);
					ovalDefinitionsHash.Add (f.Name, f.FullName);
				}
			
			ovalDefinitionsCombo.AppendText ("Specify a file or url...");
			ovalDefinitionsCombo.Active = 0;
			ovalDefinitionsCombo.Changed += new EventHandler (OnComboChanged);
			ovalDefinitionsCombo.Show ();
			
			hbox13.PackEnd (ovalDefinitionsCombo, true, true, 5);
			
			hbox22.HideAll ();
			
			if (dialogType == ActionDialogType.Scan) {
			
				computerImage.Pixbuf = Gdk.Pixbuf.LoadFromResource ("computer.png");
				systemDataLabel.HideAll ();
				systemDataFileEntry.HideAll ();
				systemDataHBox.HideAll ();
				
			} else {
			
				computerImage.Pixbuf = Gdk.Pixbuf.LoadFromResource ("edit-find.png");
			
			}
			 
			mainDialog.Icon = Gdk.Pixbuf.LoadFromResource ("sussen-icon.png");
			
			while (!scanDone)
				mainDialog.Run ();
				
			mainDialog.Destroy ();
		}
		
		void OnComboChanged (object o, EventArgs args)
		{
			ComboBox combo = o as ComboBox;
			if (o == null)
				return;
				
			TreeIter iter;
			if (combo.GetActiveIter (out iter)) {
				string selection = (string) combo.Model.GetValue (iter, 0);
				
				if (selection == "Specify a file or url...")
					hbox22.ShowAll ();
				else
					hbox22.HideAll ();
			}
		}
		
		void OnSaveResultsToggled (object o, EventArgs args)
		{
			if (saveResultsCheckButton.Active)
				ovalResultsEntry.Sensitive = true;
			else
				ovalResultsEntry.Sensitive = false;
		}
		
		void LoadPreference (String key)
		{
			object val = Preferences.Get (key);

			if (val == null)
				return;

			switch (key) {
			case Preferences.OVAL_XML_DIRECTORY:
				xmlDirectory = (string) val;
				break;

			case Preferences.OVAL_DOWNLOAD_DIRECTORY:
				downloadDirectory = (string) val;
				break;			
			}
		}
		
		void StartScan (Interpreter interpreter)
		{
			try {
		
				progressString = "Collecting data from the system...";
				interpreter.CollectData ();
				
				progressString = "Analyzing system data...";
				interpreter.AnalyzeData ();
			
			} catch (Exception e) {
			
				ShowErrorDialog ("Scan error", e.ToString());
				
			} finally {		
			
				notify.WakeupMain ();
			}
		}
		
		void ready ()
		{
			GLib.Source.Remove (scanTimer);
			notify.Close ();
			
			if (userCancelled) {
				userCancelled = false;
				return;
			}
			
			scanDone = true;
			executeButton.Sensitive = true;

			if (scanResultsFilename != null) {
				UriBuilder ub = new UriBuilder (scanResultsFilename);
				
				try {
					Gnome.Url.Show (ub.ToString());
				} catch (Exception e) {
					ShowErrorDialog ("Error displaying results", e.Message); 
				}
				
//				new SummaryDialog (scanResultsFilename);
			} else
				Console.WriteLine ("scanResultsFilename is null");
				
			mainDialog.HideAll ();
		}
		
		bool progress_timeout ()
		{
			if (scanDone)
				return false;
		
			progressBar.Pulse ();
			progressLabel.Text = progressString;

			return true;
		}
		
		void ShowErrorDialog (string title, string msg)
		{
			HIGMessageDialog dialog = new HIGMessageDialog (
							null,
							0,
							Gtk.MessageType.Error,
							Gtk.ButtonsType.Ok,
							title,
							msg);
						
			dialog.Run ();
			dialog.Destroy ();
		}
		
		string CheckFileName (string fileName)
		{
			string retVal = "";
			
			if (fileName.StartsWith ("http")) {				
				
				try {
				
					string tmp = GetTempFileName ();
					
					WebClient wc = new WebClient ();
					wc.DownloadFile (fileName, tmp);
					
					retVal = tmp;
				
				} catch (Exception e) {
				
					ShowErrorDialog ("Error downloading file", e.Message); 
				
				}
			
			} else if (fileName.StartsWith ("file:")) {

				Uri uri = new Uri (fileName.Trim());
				retVal = uri.LocalPath;
			
			} else {
				retVal = fileName;
			}
			
			return retVal;
		}
		
		static string GetTempFileName ()
		{
			string tmpFile = Path.GetTempFileName ();
			File.Delete (tmpFile);
			
			return tmpFile;
		}
		
		public void OnExecuteClicked (object o, EventArgs args)
		{
			string defFileName = null;		
		
			TreeIter iter;
			if (ovalDefinitionsCombo.GetActiveIter (out iter)) {
				string selection = (string) ovalDefinitionsCombo.Model.GetValue (iter, 0);
				if (selection == "Specify a file or url...")
					defFileName = ovalDefinitionsEntry.Text;
				else
					defFileName = (string) ovalDefinitionsHash [selection]; 
			}
	
			Config config = new Config ("linux");
			config.ResultsFileFormat = ResultOutputTypes.HTML;
			config.DefinitionsFile = CheckFileName (defFileName);
			config.XmlDirectory = xmlDirectory;
			config.DownloadDirectory = downloadDirectory;
	
			if (!File.Exists (config.DefinitionsFile)) {
				ShowErrorDialog ("File does not exist",
					String.Format ("ERROR: OVAL definitions file '{0}' does not exist.", config.DefinitionsFile));
				
				return;
			}

			if (actionDialogType == ActionDialogType.Analyze) {

				config.SystemCharacteristicsFile = systemDataFileEntry.Text;
				config.AnalyzeOnly = true;
			
				if (!File.Exists (config.SystemCharacteristicsFile)) {
					ShowErrorDialog ("File does not exist",
						String.Format ("ERROR: System data file '{0}' does not exist.", config.SystemCharacteristicsFile));

					return;				
				}
				
			} else {
								
				config.SystemCharacteristicsFile = GetTempFileName (); 

			}
			
			if (saveResultsCheckButton.Active) {
			
				if (ovalResultsEntry.Text == "") {

					ShowErrorDialog ("Invalid results file",
						String.Format ("ERROR: results file '{0}' is not valid.", ovalResultsEntry.Text));
						
					return;
				}
							
				config.ResultsFile = ovalResultsEntry.Text;				

			} else {			

				config.ResultsFile = GetTempFileName ();

			}

			scanResultsFilename = config.ResultsFile;

			Interpreter ovalInterpreter = null;
			
			try {

				ovalInterpreter = new Interpreter (config);
			
			} catch (Exception e) {
			
				ShowErrorDialog ("Error", e.Message);
				return;
			}

			executeButton.Sensitive = false;			
		
			scanTimer = GLib.Timeout.Add (100, new GLib.TimeoutHandler (progress_timeout));
		
			notify = new ThreadNotify (new ReadyEvent (ready));
			
			ThreadStart starter = new ThreadStart (delegate() {
				StartScan (ovalInterpreter);			
			});
			new Thread (starter).Start ();			
		}
		
		public void OnCancelClicked (object o, EventArgs args) 
	 	{
	 		if (!scanDone && notify != null)
	 			notify.WakeupMain ();
	 		
	 		scanDone = true;
	 		userCancelled = true;
	 	}	 	
	}
	
	// taken from Tomboy; written by Alex Graveley <alex@beatniksoftware.com>
	public class HIGMessageDialog : Gtk.Dialog
	{
		Gtk.AccelGroup accel_group;

		public HIGMessageDialog (Gtk.Window parent,
					 Gtk.DialogFlags flags,
					 Gtk.MessageType type,
					 Gtk.ButtonsType buttons,
					 string          header,
					 string          msg)
			: base ()
		{
			HasSeparator = false;
			BorderWidth = 5;
			Resizable = false;
			Title = "";

			VBox.Spacing = 12;
			ActionArea.Layout = Gtk.ButtonBoxStyle.End;

			accel_group = new Gtk.AccelGroup ();
			AddAccelGroup (accel_group);

			Gtk.HBox hbox = new Gtk.HBox (false, 12);
			hbox.BorderWidth = 5;
			hbox.Show ();
			VBox.PackStart (hbox, false, false, 0);

			Gtk.Image image = null;

			switch (type) {
			case Gtk.MessageType.Error:
				image = new Gtk.Image (Gtk.Stock.DialogError, 
						       Gtk.IconSize.Dialog);
				break;
			case Gtk.MessageType.Question:
				image = new Gtk.Image (Gtk.Stock.DialogQuestion, 
						       Gtk.IconSize.Dialog);
				break;
			case Gtk.MessageType.Info:
				image = new Gtk.Image (Gtk.Stock.DialogInfo, 
						       Gtk.IconSize.Dialog);
				break;
			case Gtk.MessageType.Warning:
				image = new Gtk.Image (Gtk.Stock.DialogWarning, 
						       Gtk.IconSize.Dialog);
				break;
			}

			image.Show ();
			hbox.PackStart (image, false, false, 0);
			
			Gtk.VBox label_vbox = new Gtk.VBox (false, 0);
			label_vbox.Show ();
			hbox.PackStart (label_vbox, true, true, 0);

			string title = String.Format ("<span weight='bold' size='larger'>{0}" +
						      "</span>\n",
						      header);

			Gtk.Label label;

			label = new Gtk.Label (title);
			label.UseMarkup = true;
			label.Justify = Gtk.Justification.Left;
			label.LineWrap = true;
			label.SetAlignment (0.0f, 0.5f);
			label.Show ();
			label_vbox.PackStart (label, false, false, 0);

			label = new Gtk.Label (msg);
			label.UseMarkup = true;
			label.Justify = Gtk.Justification.Left;
			label.LineWrap = true;
			label.SetAlignment (0.0f, 0.5f);
			label.Show ();
			label_vbox.PackStart (label, false, false, 0);
			
			switch (buttons) {
			case Gtk.ButtonsType.None:
				break;
			case Gtk.ButtonsType.Ok:
				AddButton (Gtk.Stock.Ok, Gtk.ResponseType.Ok, true);
				break;
			case Gtk.ButtonsType.Close:
				AddButton (Gtk.Stock.Close, Gtk.ResponseType.Close, true);
				break;
			case Gtk.ButtonsType.Cancel:
				AddButton (Gtk.Stock.Cancel, Gtk.ResponseType.Cancel, true);
				break;
			case Gtk.ButtonsType.YesNo:
				AddButton (Gtk.Stock.No, Gtk.ResponseType.No, false);
				AddButton (Gtk.Stock.Yes, Gtk.ResponseType.Yes, true);
				break;
			case Gtk.ButtonsType.OkCancel:
				AddButton (Gtk.Stock.Cancel, Gtk.ResponseType.Cancel, false);
				AddButton (Gtk.Stock.Ok, Gtk.ResponseType.Ok, true);
				break;
			}

			if (parent != null)
				TransientFor = parent;

			if ((int) (flags & Gtk.DialogFlags.Modal) != 0)
				Modal = true;

			if ((int) (flags & Gtk.DialogFlags.DestroyWithParent) != 0)
				DestroyWithParent = true;
		}

		void AddButton (string stock_id, Gtk.ResponseType response, bool is_default)
		{
			Gtk.Button button = new Gtk.Button (stock_id);
			button.CanDefault = true;
			button.Show ();

			AddActionWidget (button, response);

			if (is_default) {
				DefaultResponse = response;
				button.AddAccelerator ("activate",
						       accel_group,
						       (uint) Gdk.Key.Escape, 
						       0,
						       Gtk.AccelFlags.Visible);
			}
		}
	}
}
