//
// Interpreter.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Xml;
using System.Xml.Schema;

using Sussen.Oval.Apache;
using Sussen.Oval.Independent;
using Sussen.Oval.Unix;
using Sussen.Oval.Windows;

namespace Sussen.Oval
{
    public class Interpreter
    {
        Config configuration;
        OvalDocuments docs;

        DataCollector dc;
        DataAnalyzer analyzer;

        public Interpreter()
        {
        }

        public Interpreter(Config config)
        {
            if (config == null)
                throw new InterpreterException("Invalid configuration");

            if (config.DefinitionsFile == "" || config.DefinitionsFile == null)
                throw new InterpreterException("No definitions file specified");

            if (config.LogFile != "" || config.LogFile != null)
                Log.Initialize(config.LogFile, config.LoggingLevel);

            configuration = config;
            docs = new OvalDocuments();

            LoadDefinitions();
        }

        void CollectSystemData()
        {
            DateTime start;
            TimeSpan ts;

            try
            {

                docs.SystemCharacteristics.Create();

                start = DateTime.Now;
                dc = new DataCollector(docs);
                SetupDataCollector(configuration.FamilyType, dc);

                Log.Info("Examining system information");
                docs.SystemCharacteristics.WriteSystemInfo(new UnixSystemInfo());

                Log.Info("Collecting data from the system");
                dc.Run();

                ts = DateTime.Now - start;
                Log.Debug("Time taken to collect data: " + ts.ToString());

                Log.Info("Saving data to {0}", configuration.SystemCharacteristicsFile);
                docs.SystemCharacteristics.Save(configuration.SystemCharacteristicsFile);

            }
            catch (UnauthorizedAccessException uae)
            {
                Log.Error(uae);
                throw new InterpreterException(uae.Message);
            }
            catch (DirectoryNotFoundException dnfe)
            {
                Log.Error(dnfe);
                throw new InterpreterException(dnfe.Message);
            }
        }

        void LoadDefinitions()
        {
            if (configuration.VerifyMD5)
                if (!Document.CheckFileHash(configuration.DefinitionsFile, configuration.DefinitionsFileHash))
                    throw new InterpreterException(String.Format("ERROR: file '{0}' signature is not valid.", configuration.DefinitionsFile));

            if (configuration.VerifyXML)
                if (!docs.Definitions.Validate(configuration.DefinitionsFile))
                    throw new InterpreterException(String.Format("ERROR: file '{0}' cannot be validated", configuration.DefinitionsFile));

            docs.Definitions.Load(configuration.DefinitionsFile);
            if (!docs.Definitions.CheckSchemaVersion())
                throw new InterpreterException("Unsupported schema_version");
        }

        void LoadSystemData()
        {
            if (configuration.VerifyXML)
                if (!docs.SystemCharacteristics.Validate(configuration.SystemCharacteristicsFile))
                    throw new InterpreterException(String.Format("ERROR: file '{0}' cannot be validated", configuration.SystemCharacteristicsFile));

            try
            {

                Log.Info("Loading data document '{0}'", configuration.SystemCharacteristicsFile);
                docs.SystemCharacteristics.Load(configuration.SystemCharacteristicsFile);

            }
            catch (XmlException xe)
            {
                Log.Error(xe);
                throw new InterpreterException("Unable to load data document. " + xe.Message);
            }
        }

        public void CollectData()
        {
            if (!configuration.AnalyzeOnly)
                CollectSystemData();
            else
                LoadSystemData();
        }

        public void AnalyzeData()
        {
            DateTime start;
            TimeSpan ts;

            try
            {

                start = DateTime.Now;
                analyzer = new DataAnalyzer(docs);
                Log.Info("Analyzing system data");
                analyzer.Run();

                ts = DateTime.Now - start;
                Log.Debug("Time taken to analyze data: {0}", ts.ToString());

                Log.Info("Saving results to '{0}'", configuration.ResultsFile);
                docs.Results.Save(configuration);

            }
            catch (UnauthorizedAccessException uae)
            {
                Log.Error(uae);
                string msg = String.Format("Unable to save results. {0}", uae.Message);
                throw new InterpreterException(msg);
            }
            catch (DirectoryNotFoundException dnfe)
            {
                Log.Error(dnfe);
                string msg = String.Format("Unable to save results. {0}", dnfe.Message);
                throw new InterpreterException(msg);
            }
        }

        public string AnalyzeDefinition(string xml)
        {
            OvalDocuments od = new OvalDocuments();
            od.Definitions.LoadXml(xml);

            od.SystemCharacteristics.Create();

            dc = new DataCollector(od);
            dc.CreateTest = new PlatformCreateTest(UnixCreateTest);

            od.SystemCharacteristics.AddNamespace("xmlns:linux-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#linux");
            od.SystemCharacteristics.AddNamespace("xmlns:unix-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#unix");
            od.SystemCharacteristics.AddNamespace("xmlns:independent-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#independent");
            od.SystemCharacteristics.AddNamespace("xmlns:apache-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#apache");

            Log.Info("Collecting data from the system");
            od.SystemCharacteristics.WriteSystemInfo(new UnixSystemInfo());
            dc.Run();

            Log.Info("Analyzing system data");
            analyzer = new DataAnalyzer(od);
            analyzer.Run();

            return od.Results.ToString();
        }

        public static Test CreateTest(string testName)
        {
#if MONO_BUILD
            switch (testName)
            {

                case "dpkginfo_test":
                    return new DpkgInfoTest();

                case "environmentvariable_test":
                    return new EnvironmentVariableTest();

                case "family_test":
                    return new FamilyTest();

                case "file_test":
                    return new FileTest();

                case "filemd5_test":
                    return new FileMD5Test();

                case "inetd_test":
                    return new InetdTest();

                case "interface_test":
                    return new InterfaceTest();

                case "inetlisteningservers_test":
                    return new InetListeningServersTest();

                case "password_test":
                    return new PasswordTest();

                case "process_test":
                    return new ProcessTest();

                case "rpminfo_test":
                    return new RpmInfoTest();

                case "runlevel_test":
                    return new RunLevelTest();

                case "sccs_test":
                    return new SccsTest();

                case "shadow_test":
                    return new ShadowTest();

                case "textfilecontent_test":
                    return new TextFileContentTest();

                case "uname_test":
                    return new UnameTest();

                //			case "unknown_test":

                //			case "variable_test":

                case "version_test":
                    return new VersionTest();

                case "xmlfilecontent_test":
                    return new XmlFileContentTest();

                default:
                    throw new InterpreterException("Unsupport test type: " + testName);
            }
#else
            return null;
#endif
        }

        public static string RunTest(Test test, ObjectElement objectElement)
        {
            StringBuilder results = new StringBuilder();

            CollectedItem[] testResults;
            testResults = test.Run(null, objectElement);

            foreach (CollectedItem c in testResults)
            {
                results.Append(c.ToString());
                results.Append("\n");
            }

            return results.ToString();
        }

        public Config CurrentConfiguration
        {
            get { return configuration; }
            set { configuration = value; }
        }

        public string Report
        {
            get { return analyzer.ToString(); }
        }

        void SetupDataCollector(FamilyTypes familyType, DataCollector dc)
        {
            switch (familyType)
            {

                case FamilyTypes.Linux:
                    SetupLinuxDataCollector(dc);
                    break;

                case FamilyTypes.Unix:
                    SetupUnixDataCollector(dc);
                    break;

                case FamilyTypes.Windows:
                    SetupWindowsDataCollector(dc);
                    break;

                default:
                    throw new InterpreterException("Unsupported family type");
            }
        }

        void SetupLinuxDataCollector(DataCollector dc)
        {
            docs.SystemCharacteristics.AddNamespace("xmlns:linux-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#linux");
            docs.SystemCharacteristics.AddNamespace("xmlns:unix-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#unix");
            docs.SystemCharacteristics.AddNamespace("xmlns:independent-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#independent");
            docs.SystemCharacteristics.AddNamespace("xmlns:apache-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#apache");

            dc.CreateTest = new PlatformCreateTest(UnixCreateTest);
        }

        void SetupUnixDataCollector(DataCollector dc)
        {
            docs.SystemCharacteristics.AddNamespace("xmlns:unix-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#unix");
            docs.SystemCharacteristics.AddNamespace("xmlns:independent-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#independent");
            docs.SystemCharacteristics.AddNamespace("xmlns:apache-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#apache");

            dc.CreateTest = new PlatformCreateTest(UnixCreateTest);
        }

        void UnixCreateTest(TestElement testElement, ObjectElement objectElement)
        {
#if MONO_BUILD
            switch (testElement.Name)
            {

                case "dpkginfo_test":
                    dc.RunTest(new DpkgInfoTest(), testElement, objectElement);
                    break;

                case "environmentvariable_test":
                    dc.RunTest(new EnvironmentVariableTest(), testElement, objectElement);
                    break;

                case "family_test":
                    dc.RunTest(new FamilyTest(), testElement, objectElement);
                    break;

                case "file_test":
                    dc.RunTest(new FileTest(), testElement, objectElement);
                    break;

                case "filemd5_test":
                    dc.RunTest(new FileMD5Test(), testElement, objectElement);
                    break;

                case "inetd_test":
                    dc.RunTest(new InetdTest(), testElement, objectElement);
                    break;

                case "interface_test":
                    dc.RunTest(new InterfaceTest(), testElement, objectElement);
                    break;

                case "inetlisteningservers_test":
                    dc.RunTest(new InetListeningServersTest(), testElement, objectElement);
                    break;

                case "password_test":
                    dc.RunTest(new PasswordTest(), testElement, objectElement);
                    break;

                case "process_test":
                    dc.RunTest(new ProcessTest(), testElement, objectElement);
                    break;

                case "rpminfo_test":
                    dc.RunTest(new RpmInfoTest(), testElement, objectElement);
                    break;

                case "runlevel_test":
                    dc.RunTest(new RunLevelTest(), testElement, objectElement);
                    break;

                case "sccs_test":
                    dc.RunTest(new SccsTest(), testElement, objectElement);
                    break;

                case "shadow_test":
                    dc.RunTest(new ShadowTest(), testElement, objectElement);
                    break;

                case "textfilecontent_test":
                    dc.RunTest(new TextFileContentTest(), testElement, objectElement);
                    break;

                case "uname_test":
                    dc.RunTest(new UnameTest(), testElement, objectElement);
                    break;

                case "unknown_test":
                    break;

                case "variable_test":
                    dc.RunTest(new VariableTest(), testElement, objectElement);
                    break;

                case "version_test":
                    dc.RunTest(new VersionTest(), testElement, objectElement);
                    break;

                case "xmlfilecontent_test":
                    dc.RunTest(new XmlFileContentTest(), testElement, objectElement);
                    break;

                default:
                    throw new InterpreterException("Unsupport test type: " + testElement.Name);
            }
#endif
        }

        void SetupWindowsDataCollector(DataCollector dc)
        {
            docs.SystemCharacteristics.AddNamespace("xmlns:windows-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#windows");
            docs.SystemCharacteristics.AddNamespace("xmlns:independent-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#independent");
            docs.SystemCharacteristics.AddNamespace("xmlns:apache-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#apache");

            dc.CreateTest += new PlatformCreateTest(WindowsCreateTest);
        }

        void WindowsCreateTest(TestElement testElement, ObjectElement objectElement)
        {
#if WINDOWS_BUILD        
            switch (testElement.Name)
            {
                case "accesstoken_test":
                    dc.RunTest(new AccessTokenTest(), testElement, objectElement);
                    break;

                case "activedirectory_test":
                    dc.RunTest(new ActiveDirectoryTest(), testElement, objectElement);
                    break;

                case "auditeventpolicy_test":
                    dc.RunTest(new AuditEventPolicyTest(), testElement, objectElement);
                    break;

                case "environmentvariable_test":
                    dc.RunTest(new EnvironmentVariableTest(), testElement, objectElement);
                    break;

                case "family_test":
                    dc.RunTest(new FamilyTest(), testElement, objectElement);
                    break;

                case "file_test":
                    dc.RunTest(new FileTest(), testElement, objectElement);
                    break;

                case "filemd5_test":
                    dc.RunTest(new FileMD5Test(), testElement, objectElement);
                    break;

                case "fileauditedpermissions_test":
                    dc.RunTest(new FileAuditedPermissionsTest(), testElement, objectElement);
                    break;

                case "fileeffectiverights_test":
                    dc.RunTest(new FileEffectiveRightsTest(), testElement, objectElement);
                    break;

                case "group_test":
                    dc.RunTest(new GroupTest(), testElement, objectElement);
                    break;

                case "interface_test":
                    dc.RunTest(new InterfaceTest(), testElement, objectElement);
                    break;

                case "lockoutpolicy_test":
                    dc.RunTest(new LockoutPolicyTest(), testElement, objectElement);
                    break;

                case "metabase_test":
                    dc.RunTest(new MetabaseTest(), testElement, objectElement);
                    break;

                case "passwordpolicy_test":
                    dc.RunTest(new PasswordPolicyTest(), testElement, objectElement);
                    break;

                case "port_test":
                    dc.RunTest(new PortTest(), testElement, objectElement);
                    break;

                case "process_test":
                    dc.RunTest(new ProcessTest(), testElement, objectElement);
                    break;

                case "registry_test":
                    dc.RunTest(new RegistryTest(), testElement, objectElement);
                    break;

                case "regkeyauditedpermissions_test":
                    dc.RunTest(new RegKeyAuditedPermissionsTest(), testElement, objectElement);
                    break;

                case "regkeyeffectiverights_test":
                    dc.RunTest(new RegKeyEffectiveRightsTest(), testElement, objectElement);
                    break;

                case "sid_test":
                    dc.RunTest(new SidTest(), testElement, objectElement);
                    break;

                case "textfilecontent_test":
                    dc.RunTest(new TextFileContentTest(), testElement, objectElement);
                    break;

                case "unknown_test":
                    break;

                case "user_test":
                    dc.RunTest(new UserTest(), testElement, objectElement);
                    break;

                case "variable_test":
                	dc.RunTest (new VariableTest(), testElement, objectElement);
                    break;

                case "version_test":
                    dc.RunTest(new VersionTest(), testElement, objectElement);
                    break;

                case "volume_test":
                    dc.RunTest(new VolumeTest(), testElement, objectElement);
                    break;

                case "wmi_test":
                    dc.RunTest(new WMITest(), testElement, objectElement);
                    break;

                case "xmlfilecontent_test":
                    dc.RunTest(new XmlFileContentTest(), testElement, objectElement);
                    break;

                default:
                    throw new InterpreterException("Unsupport test type: " + testElement.Name);
            }
#endif
        }
    }

    public class InterpreterException : Exception
    {
        public InterpreterException()
            : base()
        {
        }

        public InterpreterException(string message)
            : base(message)
        {
        }
    }
}

#if MONO_BUILD
namespace Sussen.Oval.Windows 
{
}
#else
namespace Sussen.Oval.Unix
{
}
#endif