// 
// DpkgInfoTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct DpkgEntry
	{
		public string Name;
		public string Arch;
		public string Epoch;
		public string Release;
		public string Version;
		public string Evr;
	}	

	internal class DpkgParser
	{
		ArrayList packages;
		
		const string statusFile = "/var/lib/dpkg/status";
		const string hasEpochPattern = @"[0-9]:[0-9\.]+-[A-Za-z0-9_\-\.]*";
		
		char[] versionReleaseSep = {'-'};
		char[] epochVersionSep = {':'};
	
		public DpkgParser ()
		{
			packages = new ArrayList ();
		
			if (!File.Exists (statusFile))
				return;

			try {
			
				StreamReader sr = new StreamReader (statusFile);
				string line = null;
			
				while ((line = sr.ReadLine()) != null) {
					if (line.StartsWith ("Package:"))
						packages.Add (ReadEntry (line, sr));
				}

				sr.Close ();
				
			} catch (Exception e) {
				Log.Debug (e);
			}
		}

		DpkgEntry ReadEntry (string packageString, StreamReader sr)
		{
			char[] delim = {':'};
			string[] pairs = packageString.Split (delim, 2);		

			DpkgEntry de = new DpkgEntry ();
			de.Name  = pairs[1].Trim();
			de.Epoch = "";
			de.Release = "";
			de.Version = "";
			
			string line = null;
			while ((line = sr.ReadLine()) != null) {
				if (line.Equals ("")) {
					break;
				} else if (line.StartsWith ("Architecture")) {
					pairs = line.Split (delim, 2);
					de.Arch = pairs[1].Trim();
				} else if (line.StartsWith ("Version")) {
					string tmp = pairs[1].Trim();

					Match m = Regex.Match (tmp, hasEpochPattern);
					if (m.Success) {
						string[] eparts = tmp.Split (epochVersionSep);

						string tmpVersion = eparts[1];
						string[] vparts = tmpVersion.Split (versionReleaseSep);

						de.Epoch = eparts[0];
						de.Version = vparts[0];

						if (vparts.Length == 2)
							de.Release = vparts[1];
						
					} else {
						string[] vparts = tmp.Split (versionReleaseSep);
						de.Version = vparts[0];

						if (vparts.Length == 2)
							de.Release = vparts[1];
					}

					de.Evr = tmp;
				}
			}
			
			return de;
		}

		public DpkgEntry[] Packages
		{
			get { return (DpkgEntry[]) packages.ToArray (typeof (DpkgEntry)); }
		}
	}

	public class DpkgInfoTest : Test
	{
		DpkgParser dp;
		int itemCount;
		
		public DpkgInfoTest ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject packageName = objectElement.Entities["name"];

			if (packageName == null)
				throw new TestException (objectElement.ID, "Invalid dpkginfo test");

			if (packageName.DataType != OvalDataTypes.StringType)
				throw new TestException (objectElement.ID, "Invalid datatype for name object in dpkginfo test");

			// FIXME: Be better to share this with all test instances 
			dp = new DpkgParser ();

			if (packageName.VariableInstance != null) {
			
				foreach (string v in packageName.VariableInstance.ValueArray) {
					string[] names = GetPackageNames (v, packageName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetPackageNames (packageName.Value, packageName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] GetPackageNames (string pkgName, Operations operation)
		{		
			ArrayList items = new ArrayList ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (pkgName);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindPackages (pkgName, operation);
				
			default:
				throw new TestException ("", "Invalid operation for name object in dpkginfo test");
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindPackages (string packageName, Operations operation)
		{
			ArrayList items = new ArrayList ();

			foreach (DpkgEntry de in dp.Packages) 
				if (CompareItems (packageName, de.Name, operation))
					items.Add (de.Name);
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string packageName)
		{
			CollectedItem item = new CollectedItem ("dpkginfo_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.Complete);
			bool foundMatch = false;
		
			foreach (DpkgEntry de in dp.Packages) {
				if (de.Name == packageName) {					
					item.AddItem (new ItemEntity ("name", packageName, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("arch", de.Arch, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("epoch", de.Epoch, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("release", de.Release, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("version", de.Version, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("evr", de.Evr, OvalDataTypes.StringType, StatusFlags.Exists, false));
			
					foundMatch = true;
				}
			}
					
			if (!foundMatch) {
				item.AddItem (new ItemEntity ("name", packageName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("arch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("epoch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("release", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("version", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("evr", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			}
			
			return item;
		}		
	}
}
