// 
// FileTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Xml;
using Mono.Unix.Native;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public enum FileRecurse { None, Files, Directories, FileAndDirectories };
	public enum FileRecurseDirection { None, Up, Down };
	public enum FileRecurseFilesystem { All, Local, Defined };

	public class FileBehaviors
	{
		int maxDepth;
		FileRecurse recurse;
		FileRecurseDirection recurseDirection;
		FileRecurseFilesystem recurseFilesystem; 
	
		public FileBehaviors (Hashtable behaviors)
		{
			if (behaviors == null)
				throw new ArgumentNullException ("behaviors");
				
			if (behaviors.Count == 0)
				SetDefaults ();
			else
				SetValues (behaviors);
		}

		void SetDefaults ()
		{
			maxDepth = -1;
			recurse = FileRecurse.None;
			recurseDirection = FileRecurseDirection.None;
			recurseFilesystem = FileRecurseFilesystem.All;		
		}
		
		void SetValues (Hashtable behaviors)
		{
			string md = (string) behaviors ["max_depth"];
			if (md == null) {
				maxDepth = -1;
			} else {
				try { maxDepth = int.Parse (md); } 
				catch { maxDepth = -1; }
			}
			
			string r = (string) behaviors ["recurse"];
			if (r == null) {
				recurse = FileRecurse.None;
			} else {
				switch (r.ToLower()) {
				case "none":
					recurse = FileRecurse.None;
					break;
					
				case "files":
					recurse = FileRecurse.Files;
					break;
					
				case "directories":
					recurse = FileRecurse.Directories;
					break;
					
				case "files and directories":
					recurse = FileRecurse.FileAndDirectories;
					break;
					
				default:
					recurse = FileRecurse.None;
					break;
				}
			}
			
			string rd = (string) behaviors ["recurse_direction"];
			if (rd == null) {
				recurseDirection = FileRecurseDirection.None;
			} else {
				switch (rd.ToLower()) {
				case "none":
					recurseDirection = FileRecurseDirection.None;
					break;
					
				case "up":
					recurseDirection = FileRecurseDirection.Up;
					break;
					
				case "down":
					recurseDirection = FileRecurseDirection.Down;
					break;
					
				default:
					recurseDirection = FileRecurseDirection.None;
					break;
				}
			}
			
			string rf = (string) behaviors ["recurse_file_system"];
			if (rf == null) {
				recurseFilesystem = FileRecurseFilesystem.All;
			} else {
				switch (rf.ToLower()) {
				case "all":
					recurseFilesystem = FileRecurseFilesystem.All;
					break;
					
				case "local":
					recurseFilesystem = FileRecurseFilesystem.Local;
					break;
					
				case "defined":
					recurseFilesystem = FileRecurseFilesystem.Defined;
					break;
					
				default:
					recurseFilesystem = FileRecurseFilesystem.All;
					break;
				}
			}			
		}
		
		public int MaxDepth
		{
			get { return maxDepth; }
		}
		
		public FileRecurse Recurse
		{
			get { return recurse; } 
		}
		
		public FileRecurseDirection RecurseDirection
		{
			get { return recurseDirection; } 
		}
		
		public FileRecurseFilesystem RecurseFilesystem
		{
			get { return recurseFilesystem; } 
		}		
	}

	public class FileTest : Test
	{
		int itemCount;
		
		public FileTest ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];

			if (path == null || file == null)
				throw new TestException (objectElement.ID, "Invalid file test");

			if (file.Nil) {
			
				items.Add (CheckPath(path.Value));
				
			} else {

				FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
				FileCrawler fc = new FileCrawler (path, file, fb);
				fc.Run ();
			
				foreach (string f in fc.FoundFiles) {
					items.Add (CollectData (f));
					itemCount++;
				}
			}
				
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));					
		}

		CollectedItem CheckPath (string pathName)
		{
			DirectoryInfo di = new DirectoryInfo (pathName);
			CollectedItem item = new CollectedItem ("file_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);
							
			if (di.Exists)
				item.AddItem (new ItemEntity ("path", pathName, OvalDataTypes.StringType, StatusFlags.Exists, true));
			else
				item.AddItem (new ItemEntity ("path", pathName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				
			return item;
		}

		CollectedItem CollectData (string fullName)
		{	
			CollectedItem item = new CollectedItem ("file_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete); 

			string filePath, fileName;			
			FileInfo tmpFi = new FileInfo (fullName);
			filePath = tmpFi.Directory.FullName;
			fileName = tmpFi.Name;			
			
			try {
				
				Mono.Unix.UnixFileInfo fi = new Mono.Unix.UnixFileInfo (fullName);

				item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				
				string fileType = null;
				
				if (fi.IsFifo) {					
					fileType = "fifo";
				} else if (fi.IsCharacterDevice) {
					fileType = "character";
				} else if (fi.IsDirectory) {
					fileType = "directory";
				} else if (fi.IsBlockDevice) {
					fileType = "block special";
				} else if (fi.IsRegularFile) {		
					fileType = "regular";
				} else if (fi.IsSymbolicLink) {
					fileType = "symbolic link";
				} else if (fi.IsSocket) {
					fileType = "socket";
				}

				item.AddItem (new ItemEntity ("type", fileType, OvalDataTypes.StringType, StatusFlags.Exists, false));

				Stat fileStat = fi.ToStat (); 
				item.AddItem (new ItemEntity ("user_id", fi.OwnerUserId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("group_id", fi.OwnerGroupId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("a_time", fileStat.st_atime.ToString(), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("c_time", fileStat.st_ctime.ToString(), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("m_time", fileStat.st_mtime.ToString(), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("size", fi.Length.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));

				// Get permissions
				FilePermissions fp = fi.Protection;
				string permValue = null;

				// suid, sgid, sticky
				if ((fp & FilePermissions.S_ISUID) == FilePermissions.S_ISUID)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("suid", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));			
				
				if ((fp & FilePermissions.S_ISGID) == FilePermissions.S_ISGID)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("sgid", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));		

				if ((fp & FilePermissions.S_ISVTX) == FilePermissions.S_ISVTX)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("sticky", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
								
				// User perms
				if ((fp & FilePermissions.S_IRUSR) == FilePermissions.S_IRUSR)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("uread", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				if ((fp & FilePermissions.S_IWUSR) == FilePermissions.S_IWUSR)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("uwrite", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				if ((fp & FilePermissions.S_IXUSR) == FilePermissions.S_IXUSR)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("uexec", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				// group perms
				if ((fp & FilePermissions.S_IRGRP) == FilePermissions.S_IRGRP)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("gread", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
						
				if ((fp & FilePermissions.S_IWGRP) == FilePermissions.S_IWGRP)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("gwrite", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				if ((fp & FilePermissions.S_IXGRP) == FilePermissions.S_IXGRP)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("gexec", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
						
				// other perms
				if ((fp & FilePermissions.S_IROTH) == FilePermissions.S_IROTH)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("oread", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
							
				if ((fp & FilePermissions.S_IWOTH) == FilePermissions.S_IWOTH)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("owrite", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));		

				if ((fp & FilePermissions.S_IXOTH) == FilePermissions.S_IXOTH)
					permValue = "1";
				else
					permValue = "0";
				
				item.AddItem (new ItemEntity ("oexec", permValue, OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				return item;
				
			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				
				item.AddItem (new ItemEntity ("type", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new ItemEntity ("user_id", "", OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("group_id", "", OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("a_time", "", OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("c_time", "", OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("m_time", "", OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("size", "", OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				
				item.AddItem (new ItemEntity ("suid", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("sgid", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("sticky", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("uread", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("uwrite", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("uexec", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("gread", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("gwrite", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("gexec", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("oread", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("owrite", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("oexec", "", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				return item;
			}			
		}
	}
}
