// 
// InetListeningServersTest.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct NetstatItem
	{
		public string Protocol;
		public string LocalAddress;
		public string LocalPort;
		public string LocalFullAddress;
		public string Program;
		public string ForeignAddress;
		public string ForeignPort;
		public string ForeignFullAddress;
		public string PID;
		public string UID;
	}

	internal class NetstatParser
	{
		ArrayList netList;

		public NetstatParser ()
		{
			netList = new ArrayList ();

			string [] paths = new string[] { "/bin/netstat", "/usr/bin/netstat" };
			string line = null;
		
			ProcessStartInfo startInfo = new ProcessStartInfo ();
			startInfo.Arguments = "-p -l -t -u --numeric-ports -e";
		
			startInfo.RedirectStandardOutput = true;
			startInfo.RedirectStandardError = true;
			startInfo.UseShellExecute = false;
		
			foreach (string p in paths) {
				if (!File.Exists (p))
					continue;

				try  {			
					startInfo.FileName = p;
					Process proc = Process.Start (startInfo);

					StreamReader stdout = proc.StandardOutput;
					string output = stdout.ReadToEnd ();

					if (!proc.HasExited) {
						proc.Kill ();
						proc.Close ();
					}

					StringReader sr = new StringReader(output);

					// eat the the header
					sr.ReadLine();
					sr.ReadLine();

					while ((line = sr.ReadLine()) != null) {
						ParseLine (line);
					}

					sr.Close ();

				} catch (Exception e) {
					Log.Debug (e);
				}
			}
		}

		void ParseLine (string line)
		{
			// Proto Recv-Q Send-Q Local Address           Foreign Address         State User       Inode      PID/Program name
			// tcp        0      0 localhost:832           *:*                     LISTEN root       14845      -
			// tcp        0      0 *:389                   *:*                     LISTEN root       11631      -
			// tcp        0      0 *:5900                  *:*                     LISTEN lorenb     16279      10786/vino-server

			Regex r = new Regex (@"(\S+)(\s+)(\S+)");
			string[] split = r.Split (line);
			string[] tmp = null;

			if (split.Length < 17)
				return;

			NetstatItem ni = new NetstatItem ();
			
			tmp = SplitString (split[7], ":");
			ni.LocalAddress = tmp[0];
			ni.LocalPort = tmp[1];

			tmp = SplitString (split[9], ":");
			ni.ForeignAddress = tmp[0];
			ni.ForeignPort = tmp[1];

			ni.UID = split[13];
			ni.Protocol = split[1];
		
			string tmp2 = split[16].Trim();
			if (tmp2 == "" || tmp2 == "-") {
				ni.PID = tmp2;
				ni.Program = tmp2;
			} else {
				tmp = SplitString (split[16].Trim(), "/");
				ni.PID = tmp[0];
				ni.Program = tmp[1];
			}

			netList.Add (ni);
		}

		static string[] SplitString (string stringValue, string stringDelimiter)
		{
			string [] split = null;
			split = stringValue.Split (stringDelimiter.ToCharArray());

			return split;
		}

		public NetstatItem[] NetstatInfo
		{
			get { return (NetstatItem[]) netList.ToArray (typeof (NetstatItem)); }
		}
	}

	public class InetListeningServersTest : Test
	{
		NetstatParser np;
		int itemCount;
		
		public InetListeningServersTest ()
		{
			itemCount = 0;
		}		
		
		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject protocol = objectElement.Entities["protocol"];
			EntityObject localAddress = objectElement.Entities["local_address"];
			EntityObject localPort = objectElement.Entities["local_port"];

			if (localAddress == null || protocol == null || localPort == null)
				throw new TestException (testElement.ID, "Invalid inetlisteningservers test");

			np = new NetstatParser ();
			
			string[] protocols = GetProtocols (protocol);
			string[] addresses = GetLocalAddresses (localAddress);			
			string[] ports = GetLocalPorts (localPort);
			
			foreach (string p in protocols) {
				foreach (string a in addresses)
					foreach (string b in ports)
						items.Add (CollectData (p, a, b));
			}
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] GetLocalAddresses (EntityObject localAddress)
		{
			ArrayList items = new ArrayList ();

			if (localAddress.VariableInstance != null) {
			
				foreach (string v in localAddress.VariableInstance.ValueArray) {
				
					switch (localAddress.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;

					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] foundItems = FindLocalAddresses (v, localAddress.Operation);
						foreach (string f in foundItems)
							items.Add (f);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (localAddress.Operation));
					}		
				}
				
			} else {

				switch (localAddress.Operation) {
				
				case Operations.Equals:
					items.Add (localAddress.Value);
					break;

				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] foundItems = FindLocalAddresses (localAddress.Value, localAddress.Operation);
					foreach (string f in foundItems)
						items.Add (f);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (localAddress.Operation));
				}			
			
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] GetProtocols (EntityObject protocol)
		{
			ArrayList items = new ArrayList ();

			if (protocol.VariableInstance != null) {
			
				foreach (string v in protocol.VariableInstance.ValueArray) {				
					switch (protocol.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;

					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] foundItems = FindProtocols (v, protocol.Operation);
						foreach (string f in foundItems)
							items.Add (f);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (protocol.Operation));
					}				
				}
				
			} else {

				switch (protocol.Operation) {
				
				case Operations.Equals:
					items.Add (protocol.Value);
					break;

				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] foundItems = FindProtocols (protocol.Value, protocol.Operation);
					foreach (string f in foundItems)
						items.Add (f);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (protocol.Operation));
				}			
			
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] GetLocalPorts (EntityObject localPort)
		{
			ArrayList items = new ArrayList ();

			if (localPort.VariableInstance != null) {
			
				foreach (string v in localPort.VariableInstance.ValueArray) {
					switch (localPort.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;

					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] foundItems = FindLocalPorts (v, localPort.Operation);
						foreach (string f in foundItems)
							items.Add (f);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (localPort.Operation));
					}				
				}
			
			} else {

				switch (localPort.Operation) {
				
				case Operations.Equals:
					items.Add (localPort.Value);
					break;

				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] foundItems = FindLocalPorts (localPort.Value, localPort.Operation);
					foreach (string f in foundItems)
						items.Add (f);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (localPort.Operation));
				}			
			
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindLocalAddresses (string itemName, Operations itemOperation)
		{
			ArrayList items = new ArrayList ();

			foreach (NetstatItem ni in np.NetstatInfo)
				if (CompareItems (itemName, ni.LocalAddress, itemOperation))
					items.Add (ni.LocalAddress);
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindProtocols (string itemName, Operations itemOperation)
		{
			ArrayList items = new ArrayList ();

			foreach (NetstatItem ni in np.NetstatInfo)
				if (CompareItems (itemName, ni.Protocol, itemOperation))
					items.Add (ni.Protocol);
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindLocalPorts (string itemName, Operations itemOperation)
		{
			ArrayList items = new ArrayList ();

			foreach (NetstatItem ni in np.NetstatInfo)
				if (CompareItems (itemName, ni.LocalPort, itemOperation))
					items.Add (ni.LocalPort);
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string protocol, string address, string port)
		{
			CollectedItem item = new CollectedItem ("inetlisteningservers_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.Complete);	
			bool foundMatch = false;
			
			foreach (NetstatItem ni in np.NetstatInfo) {
				if (protocol == ni.Protocol && address == ni.LocalAddress && port == ni.LocalPort) {
					// FIXME: Collect items: local_full_address, foreign_full_address
					item.AddItem (new ItemEntity ("protocol", ni.Protocol, OvalDataTypes.IntegerType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("local_address", ni.LocalAddress, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("local_port", ni.LocalPort, OvalDataTypes.IntegerType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("program_name", ni.Program, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("pid", ni.PID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("local_full_address", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
					item.AddItem (new ItemEntity ("foreign_full_address", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
					item.AddItem (new ItemEntity ("foreign_address", ni.ForeignAddress, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("foreign_port", ni.ForeignPort, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("user_id", ni.UID, OvalDataTypes.StringType, StatusFlags.Exists, false));

					foundMatch = true;
				}
			}
			
			if (!foundMatch) {
					item.AddItem (new ItemEntity ("protocol", protocol, OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, true));
					item.AddItem (new ItemEntity ("local_address", address, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
					item.AddItem (new ItemEntity ("local_port", port, OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, true));
					item.AddItem (new ItemEntity ("program_name", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new ItemEntity ("pid", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new ItemEntity ("local_full_address", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new ItemEntity ("foreign_full_address", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new ItemEntity ("foreign_address", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new ItemEntity ("foreign_port", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new ItemEntity ("user_id", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));			
			}
					
			return item;
		}
	}
}
