// 
// InetdTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct InetdItem
	{
		public string ServiceName;
		public string EndPointType;
		public string Protocol;
		public string WaitStatus;
		public string UID;
		public string ServerProgram;
		public string ServerArgs;
	}

	internal class InetdParser
	{
		ArrayList services;
		
		public InetdParser (string fileName)
		{
			services = new ArrayList ();
			
			if (!File.Exists (fileName))
				return;
			
			try {
			
				string line = null;
				StreamReader sr = new StreamReader (fileName);
				while ((line = sr.ReadLine()) != null) {
					if (line.StartsWith ("#"))
						continue;

					char[] splitter  = {'\t',' '};
					string[] fields = line.Split (splitter, 7);
					
					InetdItem ii = new InetdItem ();
					ii.ServiceName = fields[0].Trim();
					ii.EndPointType = fields[1].Trim();
					ii.Protocol = fields[2].Trim();
					ii.WaitStatus = fields[3].Trim();
					ii.UID = fields[4].Trim();
					ii.ServerProgram = fields[5].Trim();
					
					if (fields.Length == 7)
						ii.ServerArgs = fields[6].Trim();
						
					services.Add (ii);
				}
				
				sr.Close ();
				
			} catch (Exception e) {
				Log.Debug (e);
			}
		}
		
		public InetdItem[] Services
		{
			get { return (InetdItem[]) services.ToArray (typeof (InetdItem)); }  
		}
	}

	public class InetdTest : Test
	{
		InetdParser ip;
		int itemCount;
		
		public InetdTest ()
		{
			itemCount = 0; 
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			ip = new InetdParser ("/etc/inetd.conf");
			
			EntityObject protocol = objectElement.Entities["protocol"];
			EntityObject serverProgram = objectElement.Entities["server_program"];

			if (serverProgram == null || protocol == null)
				throw new TestException (objectElement.ID, "Invalid inetd test");

			string[] protoNames = GetProtocolNames (protocol);
			string[] programNames = GetPrograms (serverProgram);

			foreach (string p in protoNames) {
				foreach (string s in programNames) {
					items.Add (CollectData (p, s));
				}
			}

			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] GetProtocolNames (EntityObject protocol)
		{
			ArrayList items = new ArrayList ();

			if (protocol.VariableInstance != null) {
			
				foreach (string v in protocol.VariableInstance.ValueArray) {
				
					switch (protocol.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;

					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] protos = FindProtocolNames (v, protocol.Operation);
						foreach (string p in protos)
							items.Add (p);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (protocol.Operation));
					}				
				}
			
			} else {

				switch (protocol.Operation) {
				
				case Operations.Equals:
					items.Add (protocol.Value);
					break;

				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] protos = FindProtocolNames (protocol.Value, protocol.Operation);
					foreach (string p in protos)
						items.Add (p);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (protocol.Operation));
				}
			
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] GetPrograms (EntityObject serverProgram)
		{
			ArrayList items = new ArrayList ();

			if (serverProgram.VariableInstance != null) {
			
				foreach (string v in serverProgram.VariableInstance.ValueArray) {
				
					switch (serverProgram.Operation) {
		
					case Operations.Equals:
						items.Add (v);
						break;
						
					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] progs = FindPrograms (v, serverProgram.Operation);
						foreach (string p in progs)
							items.Add (p);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (serverProgram.Operation));
					}
				
				}
				
			} else {

				switch (serverProgram.Operation) {
				
				case Operations.Equals:
					items.Add (serverProgram.Value);
					break;
					
				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] progs = FindPrograms (serverProgram.Value, serverProgram.Operation);
					foreach (string p in progs)
						items.Add (p);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (serverProgram.Operation));
				}			
			
			}
					
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindProtocolNames (string protocol, Operations operation)
		{
			ArrayList items = new ArrayList ();

			foreach (InetdItem ii in ip.Services) 
				if (CompareItems (protocol, ii.Protocol, operation))
					items.Add (ii.Protocol);
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindPrograms (string serverProgram, Operations operation)
		{
			ArrayList items = new ArrayList ();

			foreach (InetdItem ii in ip.Services) 
				if (CompareItems (serverProgram, ii.ServerProgram, operation))
					items.Add (ii.ServerProgram);
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string protocol, string serverProgram)
		{
			CollectedItem item = new CollectedItem ("inetd_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);
			bool foundMatch = false;
		
			foreach (InetdItem ii in ip.Services) {
				if (ii.Protocol == protocol && ii.ServerProgram == serverProgram) {					
					item.AddItem (new ItemEntity ("protocol", protocol, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("server_program", serverProgram, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("endpoint_type", ii.EndPointType, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("exec_as_user", ii.UID , OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("service_name", ii.ServiceName, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("server_arguments", ii.ServerArgs , OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("wait_status", ii.WaitStatus, OvalDataTypes.StringType, StatusFlags.Exists, false));

					foundMatch = true;
				}
			}
					
			if (!foundMatch) {
				item.AddItem (new ItemEntity ("protocol", protocol, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("server_program", serverProgram, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("endpoint_type", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("exec_as_user", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("service_name", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("server_arguments", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("wait_status", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));			
			}
					
			return item;		
		}
	}
}
