// 
// ProcessTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct ProcessItem
	{
		public string Program;
		public string UID;
		public string PID;
		public string PPID;
		public string Start;
		public string TTY;
		public string Etime;
	}

	internal class PsParser
	{
		ArrayList processList;

		public PsParser ()
		{
			processList = new ArrayList ();

			string [] paths = new string[] { "/bin/ps", "/usr/bin/ps" };
			string line = null;
		
			ProcessStartInfo startInfo = new ProcessStartInfo ();
			startInfo.Arguments = "-ef";
		
			startInfo.RedirectStandardOutput = true;
			startInfo.RedirectStandardError = true;
			startInfo.UseShellExecute = false;
		
			foreach (string p in paths) {
				if (!File.Exists (p))
					continue;

				try  {
				
					startInfo.FileName = p;
					Process proc = Process.Start (startInfo);
					StreamReader stdout = proc.StandardOutput;

					// eat the ps header
					stdout.ReadLine();

					while ((line = stdout.ReadLine()) != null) {
						ParseLine (line);
					}

					if (!proc.HasExited) {
						proc.Kill ();
						proc.Close ();
					}

				} catch (Exception e) {
					Log.Debug (e);
				}
			}
		}

		void ParseLine (string line)
		{
			// UID        PID  PPID  C STIME TTY          TIME CMD
			// root      4785     1  0 Dec20 ?        00:00:00 /sbin/udevd --daemon

			Regex r = new Regex (@"(\S+)(\s+)(\S+)");
			string[] split = r.Split (line, 8);

			ProcessItem pi = new ProcessItem ();
			pi.UID = split[1];
			pi.PID = split[3];
			pi.PPID = split[5];
			pi.Start = split[9];
			pi.TTY = split[11];
			pi.Etime = split[13];
			pi.Program = split[15];
			
			processList.Add (pi);
		}

		public ProcessItem[] ProcessList
		{
			get { return (ProcessItem[]) processList.ToArray (typeof (ProcessItem)); }
		}
	}

	public class ProcessTest : Test
	{
		PsParser ps;
		int itemCount;
		
		public ProcessTest ()
		{
			itemCount = 0;
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject cmd = objectElement.Entities["command"];

			if (cmd == null)
				throw new TestException (objectElement.ID, "Invalid process test");

			ps = new PsParser ();

			if (cmd.VariableInstance != null) {
			
				foreach (string v in cmd.VariableInstance.ValueArray) {
					string[] names = GetProcessNames (v, cmd.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetProcessNames (cmd.Value, cmd.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] GetProcessNames (string name, Operations operation)
		{		
			ArrayList items = new ArrayList ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindProcesses (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for command object in process test");
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindProcesses (string cmd, Operations operation)
		{
			ArrayList items = new ArrayList ();

			foreach (ProcessItem pi in ps.ProcessList) 
				if (CompareItems (cmd, pi.Program, operation))
					items.Add (pi.Program);
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string cmd)
		{
			CollectedItem item = new CollectedItem ("process_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);
			bool foundMatch = false;
			
			foreach (ProcessItem pi in ps.ProcessList) {
				if (cmd == pi.Program) {
					// FIXME: get scheduling_class and priority data items
					item.AddItem (new ItemEntity ("command", pi.Program, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("scheduling_class", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
					item.AddItem (new ItemEntity ("user_id", pi.UID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("pid", pi.PID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("ppid", pi.PPID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("priority", "", OvalDataTypes.IntegerType, StatusFlags.NotCollected, false));
					item.AddItem (new ItemEntity ("start_time", pi.Start, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("tty", pi.TTY, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new ItemEntity ("exec_time", pi.Etime, OvalDataTypes.StringType, StatusFlags.Exists, false));
					
					foundMatch = true;
				}
			}
			
			if (!foundMatch) {
				item.AddItem (new ItemEntity ("command", cmd, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("scheduling_class", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("user_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("pid", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("ppid", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("priority", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("start_time", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("tty", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("exec_time", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));			
			}
			
			return item;
		}
	}
}
