// 
// RpmInfoTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Xml;
using Mono.Unix;
using Mono.Unix.Native;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class RpmInfoTest : Test
	{
		int itemCount;
	
		public RpmInfoTest ()
		{
			itemCount = 0;
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject rpmName = objectElement.Entities["name"];

			if (rpmName == null)
				throw new TestException (objectElement.ID, "Invalid rpminfo test");

			if (rpmName.VariableInstance != null) {
			
				foreach (string v in rpmName.VariableInstance.ValueArray) {
					string[] names = GetPackageNames (v, rpmName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetPackageNames (rpmName.Value, rpmName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
		
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] GetPackageNames (string pkgName, Operations operation)
		{		
			ArrayList items = new ArrayList ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (pkgName);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindPackages (pkgName, operation);
				
			default:
				throw new TestException ("", "Invalid operation for name object in rpminfo test");
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindPackages (string packageName, Operations operation)
		{
			ArrayList items = new ArrayList ();

			// FIXME: Find better way to get list of package names.
			string tmp = Path.GetTempFileName ();
			rpm_dump_package_names (tmp);

			try {
							
				StreamReader sr = new StreamReader (tmp);
				string line = null;
			
				while ((line = sr.ReadLine()) != null) {
					if (CompareItems (packageName, line, operation))
						items.Add (line);
				}

				sr.Close ();
				
			} catch (Exception e) {
				Log.Debug (e);
			}

			File.Delete (tmp);

			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string rpmName)
		{
			CollectedItem item = new CollectedItem ("rpminfo_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.NotCollected);			
			_RpmInfo ri;
			
			int rc = rpm_get_info (rpmName, out ri);

			if (rc >= 0) {		

				string epoch = UnixMarshal.PtrToString (ri.rpm_epoch);
				string release = UnixMarshal.PtrToString (ri.rpm_release);
				string version = UnixMarshal.PtrToString (ri.rpm_version);

				item.Flag = Flags.Complete;
				item.AddItem (new ItemEntity ("name", rpmName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new ItemEntity ("arch", UnixMarshal.PtrToString (ri.rpm_arch), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("epoch", epoch, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("release", release, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("version", version, OvalDataTypes.StringType, StatusFlags.Exists, false));
				
				string evr = "";
				if (epoch == "-1")
					evr = String.Format ("{0}-{1}", version, release);
				else
					evr = String.Format ("{0}:{1}-{2}", epoch, version, release);
				
				item.AddItem (new ItemEntity ("evr", evr, OvalDataTypes.StringType, StatusFlags.NotCollected, false));
				
				// FIXME: don't know how to get this from the rpm database
				item.AddItem (new ItemEntity ("signature_keyid", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
										
			} else {

				item.Flag = Flags.Complete;
				item.AddItem (new ItemEntity ("name", rpmName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("arch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("epoch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("release", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("version", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));					
				item.AddItem (new ItemEntity ("evr", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("signature_keyid", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				
			}
			
			return item;
		}
		
		private struct _RpmInfo {
			public IntPtr	rpm_name;
			public IntPtr 	rpm_epoch;
			public IntPtr	rpm_version;
			public IntPtr	rpm_release;
			public IntPtr	rpm_arch;
		}
		
		[DllImport("libsussensupport")]
		static extern int rpm_get_info (string packageName, out _RpmInfo ri);
		
		[DllImport("libsussensupport")]
		static extern int rpm_dump_package_names (string fileName);
	}
}
