// 
// RunLevelTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class RunLevelTest : Test
	{
		int itemCount;
		
		public RunLevelTest ()
		{
			itemCount = 0;
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject serviceName = objectElement.Entities["service_name"];
			EntityObject runlevel = objectElement.Entities["runlevel"];

			if (serviceName == null || runlevel == null)
				throw new TestException (objectElement.ID, "Invalid runlevel test");

			string[] snames = GetServiceNames (serviceName);
			string[] runlevels = GetRunLevels (runlevel);

			foreach (string s in snames) {
				foreach (string r in runlevels) {
					items.Add (CollectData (s, r));
				}
			}
					
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}
		
		static string[] GetServiceNames (EntityObject serviceName)
		{
			ArrayList items = new ArrayList ();

			if (serviceName.VariableInstance != null) {
			
				foreach (string v in serviceName.VariableInstance.ValueArray) {				
					switch (serviceName.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;

					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] dirs = FindServiceNames (v, serviceName.Operation);
						foreach (string d in dirs)
							items.Add (d);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (serviceName.Operation));
					}				
				}
			
			} else {
			
				switch (serviceName.Operation) {
				
				case Operations.Equals:
					items.Add (serviceName.Value);
					break;

				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] dirs = FindServiceNames (serviceName.Value, serviceName.Operation);
					foreach (string d in dirs)
						items.Add (d);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (serviceName.Operation));
				}
			
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		static string[] GetRunLevels (EntityObject runLevel)
		{
			ArrayList items = new ArrayList ();

			if (runLevel.VariableInstance != null) {
			
				foreach (string v in runLevel.VariableInstance.ValueArray) {
				
					switch (runLevel.Operation) {
					case Operations.Equals:
						items.Add (v);
						break;
						
					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] dirs = FindRunLevels (v, runLevel.Operation);
						foreach (string d in dirs)
							items.Add (d);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (runLevel.Operation));
					}			
				}
				
			} else {
			
				switch (runLevel.Operation) {
				
				case Operations.Equals:
					items.Add (runLevel.Value);
					break;
					
				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] dirs = FindRunLevels (runLevel.Value, runLevel.Operation);
					foreach (string d in dirs)
						items.Add (d);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (runLevel.Operation));
				}			
			
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		static string[] FindServiceNames (string serviceName, Operations serviceOperation)
		{
			ArrayList items = new ArrayList ();

			string filePath = "/etc/init.d";
			string[] files = Directory.GetFiles (filePath);
			foreach (string f in files)
				if (CompareItems (serviceName, f, serviceOperation))
					items.Add (f);
			
			return (string[]) items.ToArray (typeof (string));
		}

		static string[] FindRunLevels (string runLevel, Operations runLevelOperation)
		{
			ArrayList items = new ArrayList ();

			string filePath = "/etc";
			string[] dirs = Directory.GetDirectories (filePath);
			foreach (string d in dirs)
				if (CompareItems (runLevel, d, runLevelOperation))
					items.Add (d);
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string serviceName, string runlevel)
		{
			CollectedItem item = new CollectedItem ("runlevel_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.NotCollected); 

			StringBuilder ip = new StringBuilder ();
			ip.AppendFormat ("/etc/init.d/{0}", serviceName);

			if (!File.Exists (ip.ToString())) {
				item.Flag = Flags.DoesNotExist;
				item.AddItem (new ItemEntity ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("start", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("kill", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				
				return item;
			}

			// FIXME: Going to be an issue for found runlevels
			StringBuilder rcp = new StringBuilder ();
			rcp.AppendFormat ("/etc/rc{0}.d", runlevel);

			DirectoryInfo dir = new DirectoryInfo (rcp.ToString());
			if (!dir.Exists) {
				item.Flag = Flags.DoesNotExist;
				item.AddItem (new ItemEntity ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("start", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("kill", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				
				return item;
			}

			bool allGood = false;

			foreach (FileInfo f in dir.GetFiles("*")) {
				string exp = "[SK][0-9][0-9]" + serviceName;
				Regex r = new Regex (exp);
				Match m = r.Match (f.Name);

				if (m.Success) {

					item.Flag = Flags.Complete;
					item.AddItem (new ItemEntity ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
					item.AddItem (new ItemEntity ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
					
					if (f.Name.StartsWith ("S")) {
						item.AddItem (new ItemEntity ("start", "1", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
						item.AddItem (new ItemEntity ("kill", "0", OvalDataTypes.BooleanType, StatusFlags.Exists, false));					
					} else {
						item.AddItem (new ItemEntity ("start", "0", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
						item.AddItem (new ItemEntity ("kill", "1", OvalDataTypes.BooleanType, StatusFlags.Exists, false));					
					}

					allGood = true;
				}
			}

			if (!allGood) {
				item.Flag = Flags.DoesNotExist;
				item.AddItem (new ItemEntity ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("start", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("kill", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			}			
			
			return item;
		}
	}
}
