// 
// Analyzer.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.Xml;

namespace Sussen.Oval
{
	public class Analyzer
	{
		protected Hashtable resultsCache;
	
		public Analyzer ()
		{
			resultsCache = new Hashtable ();
		}

		public static TestResult AnalyzeResults (ArrayList results, string operatorName)
		{
			TestResult result = TestResult.Unknown;
		
			if (operatorName != "AND" && operatorName != "OR" && operatorName != "XOR") {
				return result;
			}
		
			if (results.Count == 0)
				return result;
				
			int trueCount, falseCount, errorCount, unknownCount, notEvaluatedCount, notApplicableCount;
			GetResultsCount (results, out trueCount, out falseCount, out errorCount, out unknownCount, out notEvaluatedCount, out notApplicableCount);
		
			TestResult combinedResult = TestResult.Unknown;
		
			if (operatorName == "AND") {
			
				if (trueCount > 0 && falseCount == 0 && errorCount == 0 && unknownCount == 0 && notEvaluatedCount == 0 && notApplicableCount == 0)
					combinedResult = TestResult.True;
				else if (falseCount > 0)
					combinedResult = TestResult.False;
				else if (errorCount > 0)
					combinedResult = TestResult.Error;
				else if (unknownCount > 0)
					combinedResult = TestResult.Unknown;
				else if (notEvaluatedCount > 0)
					combinedResult = TestResult.NotEvaluated;
				else
					combinedResult = TestResult.NotApplicable;
			
			} else if (operatorName == "OR") {

				if (trueCount > 0)
					combinedResult = TestResult.True;
				else if (falseCount > 0)
					combinedResult = TestResult.False;
				else if (errorCount > 0)
					combinedResult = TestResult.Error;
				else if (unknownCount > 0)
					combinedResult = TestResult.Unknown;
				else if (notEvaluatedCount > 0)
					combinedResult = TestResult.NotEvaluated;
				else
					combinedResult = TestResult.NotApplicable;					
			
			} else if (operatorName == "XOR") {

				if (errorCount > 0)
					combinedResult = TestResult.Error;
				else if (unknownCount > 0)
					combinedResult = TestResult.Unknown;
				else if (notEvaluatedCount > 0)
					combinedResult = TestResult.NotEvaluated;
				else if (notApplicableCount > 0)
					combinedResult = TestResult.NotApplicable;
				else if ((trueCount % 2) == 1)
					combinedResult = TestResult.True;
				else
					combinedResult = TestResult.False;
			}
		
			return combinedResult;
		}

		public static TestResult AnalyzeResultsByCheck (ArrayList results, string checkName)
		{
			TestResult result = TestResult.Unknown;

			if (checkName == null)
				return result;

			if (results.Count == 0)
				return result;

			int trueCount, falseCount, errorCount, unknownCount, notEvaluatedCount, notApplicableCount;
			GetResultsCount (results, out trueCount, out falseCount, out errorCount, out unknownCount, out notEvaluatedCount, out notApplicableCount);
			
			TestResult combinedResult = TestResult.Unknown;

			switch (checkName) {
			
			case "all":
				if (trueCount == results.Count)
					combinedResult = TestResult.True;
				else if (falseCount >= 1)
					combinedResult = TestResult.False;
				else if (errorCount > 0)
					combinedResult = TestResult.Error;
				else if (unknownCount > 0)
					combinedResult = TestResult.Unknown;
				else if (notEvaluatedCount > 0)
					combinedResult = TestResult.NotEvaluated;
				else
					combinedResult = TestResult.NotApplicable;					
				break;
				
			case "at least one":
				if (trueCount >= 1)
					combinedResult = TestResult.True;
				else if (trueCount == 0 && errorCount == 0 && unknownCount == 0 && notEvaluatedCount == 0 && notApplicableCount == 0)
					combinedResult = TestResult.False;
				else if (errorCount > 0)
					combinedResult = TestResult.Error;
				else if (unknownCount > 0)
					combinedResult = TestResult.Unknown;
				else if (notEvaluatedCount > 0)
					combinedResult = TestResult.NotEvaluated;
				else
					combinedResult = TestResult.NotApplicable;					
				break;

			case "only one":
				if (trueCount == 1 && errorCount == 0 && unknownCount == 0 && notEvaluatedCount == 0)
					combinedResult = TestResult.True;
				else if (trueCount > 1)
					combinedResult = TestResult.False;
				else if (trueCount == 0 && errorCount == 0 && unknownCount == 0 && notEvaluatedCount == 0 && notApplicableCount == 0)
					combinedResult = TestResult.False;					
				else if (errorCount > 0)
					combinedResult = TestResult.Error;
				else if (unknownCount > 0)
					combinedResult = TestResult.Unknown;
				else if (notEvaluatedCount > 0)
					combinedResult = TestResult.NotEvaluated;
				else
					combinedResult = TestResult.NotApplicable;					
				break;

				
			case "none":
				if (trueCount == 0 && errorCount == 0 && unknownCount == 0 && notEvaluatedCount == 0 && notApplicableCount == 0)
					combinedResult = TestResult.True;
				else if (trueCount >= 1)
					combinedResult = TestResult.False;
				else if (trueCount == 0 && errorCount > 0)
					combinedResult = TestResult.Error;					
				else if (trueCount == 0 && unknownCount > 0)
					combinedResult = TestResult.Unknown;
				else if (trueCount == 0 && notEvaluatedCount > 0)
					combinedResult = TestResult.NotEvaluated;
				else
					combinedResult = TestResult.NotApplicable;
				break;
			
            case "none exist":
                break;

			default:
				throw new ArgumentOutOfRangeException ("Invalid check name: " + checkName);
			}
				
			return combinedResult;			
		}

		public static TestResult NegateResult (TestResult result)
		{
			switch (result) {
			
			case TestResult.True:
				return TestResult.False;
				
			case TestResult.False:
				return TestResult.True;

			case TestResult.Error:
				return TestResult.Error;
				
			case TestResult.Unknown:
				return TestResult.Unknown;

			case TestResult.NotEvaluated:
				return TestResult.NotEvaluated;

			case TestResult.NotApplicable:
				return TestResult.NotApplicable;
							
			default:
				throw new ArgumentOutOfRangeException ("Invalid test result");
			}
		}

		static void GetResultsCount (ArrayList results, out int trueCount, out int falseCount, out int errorCount, out int unknownCount,
							 out int notEvaluatedCount, out int notApplicableCount)
		{
			trueCount = 0;
			falseCount = 0;
			errorCount = 0;
			unknownCount = 0;
			notEvaluatedCount = 0;
			notApplicableCount = 0;
			
			foreach (int i in results) {
				switch (i) {
				
				case (int)TestResult.True:
					trueCount++;
					break;
					
				case (int)TestResult.False:
					falseCount++;
					break;
					
				case (int)TestResult.Error:
					errorCount++;
					break;
					
				case (int)TestResult.Unknown:
					unknownCount++;
					break;
					
				case (int)TestResult.NotEvaluated:
					notEvaluatedCount++;
					break;
					
				case (int)TestResult.NotApplicable:
					notApplicableCount++;
					break;					
				}				
			}		
		}
	}
	
	public class AnalyzerException : Exception
	{
		public AnalyzerException () : base ()
		{
		}

		public AnalyzerException (string message) : base (message)
		{
		}
	}
}
