// 
// DefinitionAnalyzer.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval
{
	public class DefinitionAnalyzer : Analyzer
	{
		OvalDocuments docs;
		XmlElement ovalDefinitionElement;
		XmlElement definitionElement;
		XmlElement criteriaElement;
			
		public DefinitionAnalyzer (XmlElement ovalDefinition, OvalDocuments ovalDocuments)
		{
			if (ovalDefinition == null)
				throw new ArgumentNullException ("ovalDefinition");

			if (ovalDocuments == null)
				throw new ArgumentNullException ("ovalDocuments");
		
			docs = ovalDocuments;
			ovalDefinitionElement = ovalDefinition;
		
			string id = ovalDefinitionElement.GetAttribute ("id");
			Match m = Regex.Match (id, Vocabulary.DEFINITION_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for definition ID", "id");
			}			
		
			definitionElement = docs.Results.CreateElement ("definition");
			definitionElement.SetAttribute ("definition_id", id);
			definitionElement.SetAttribute ("version", ovalDefinition.GetAttribute ("version"));
			// FIXME: Doesn't handle analyzing the same definition multiple times with different values
			definitionElement.SetAttribute ("variable_instance", "1");
			
			criteriaElement = docs.Results.CreateElement ("criteria");
			definitionElement.AppendChild (criteriaElement);			
		}
		
		public TestResult Run ()
		{
			XmlNodeList criteriaList = ovalDefinitionElement.GetElementsByTagName ("criteria");
			if (!(criteriaList.Count > 0))
				throw new AnalyzerException ("No criteria found in definition");
				
			Log.Debug ("Number of criteria found: {0}", criteriaList.Count);

			XmlElement ce = (XmlElement) criteriaList[0];			
			TestResult criteriaResult = AnalyzeCriteria (ce, criteriaElement);
			
			Log.Debug ("Test result after AnalyzeCriteria: {0}", Vocabulary.GetTestResult (criteriaResult));
			
			definitionElement.SetAttribute ("result", Vocabulary.GetTestResult (criteriaResult));

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				throw new AnalyzerException ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				throw new AnalyzerException ("No tests element found in results document");
				
			XmlElement definitions = (XmlElement) nl[0];
			definitions.AppendChild (definitionElement);
			
			return criteriaResult;
		}

		TestResult AnalyzeCriteria (XmlElement criteriaElement, XmlElement resCriteriaElement)
		{
			ArrayList allResults = new ArrayList ();			
			TestResult criteriaResult = TestResult.Unknown;
			string criteriaOperator = criteriaElement.GetAttribute ("operator");
			
			foreach (XmlElement e in criteriaElement.ChildNodes) {
		
				if (e.Name == "criteria") {
					XmlElement newResCritElement = docs.Results.CreateElement ("criteria");
					resCriteriaElement.AppendChild (newResCritElement);					
					TestResult result = AnalyzeCriteria (e, newResCritElement);
					allResults.Add (result);
				} else if (e.Name == "criterion") {				
					TestResult result = AnalyzeCriterion (e, resCriteriaElement);
					allResults.Add (result);
				} else if (e.Name == "extend_definition") {
					DefinitionAnalyzer da = new DefinitionAnalyzer (e, docs);
					TestResult result = da.Run ();
					allResults.Add (result);					
				}
			}
			
			criteriaResult = AnalyzeResults (allResults, criteriaOperator);
			
			resCriteriaElement.SetAttribute ("operator", criteriaOperator);
			resCriteriaElement.SetAttribute ("result", Vocabulary.GetTestResult (criteriaResult)); 			
			
			Log.Debug ("Analyzed definition criteria; operator: [{0}] result: [{1}]", criteriaOperator, Vocabulary.GetTestResult (criteriaResult)); 
			
			return criteriaResult;
		}

		TestResult AnalyzeCriterion (XmlElement criterionElement, XmlElement resCritElement)
		{
			TestResult criterionResult = TestResult.Unknown;
			string testRef = criterionElement.GetAttribute ("test_ref");
			string testNegate = criterionElement.GetAttribute ("negate");
			
			if (testRef == null)
				throw new AnalyzerException ("Unable to find test reference in criterion element");

			TestElement testElement = docs.Definitions.Tests [testRef];				
			if (testElement == null)
				throw new AnalyzerException ("Unable to find test element " + testRef);
			
			if (resultsCache.Contains (testRef)) {
				criterionResult = (TestResult) resultsCache [testRef];			
				Log.Debug ("Got cached result {0} for test {1}", criterionResult, testRef);				
			} else {
				TestAnalyzer ta = new TestAnalyzer ();
				criterionResult = ta.Run (testElement, docs);
				
				if (testNegate == "true")
					criterionResult = NegateResult (criterionResult);
					
				resultsCache.Add (testRef, criterionResult);
			}

			Log.Debug ("Analyzed definition criterion; testRef: [{0}] negate: [{1}]", testRef, testNegate);

			WriteCriterion (testRef, testNegate, criterionResult, resCritElement);
			
			return criterionResult;
		}

		void WriteCriterion (string testRef, string testNegate, TestResult result, XmlElement resCritElement)
		{			
			XmlElement criterion = docs.Results.CreateElement ("criterion");
			criterion.SetAttribute ("test_ref", testRef);
			criterion.SetAttribute ("version", "1");
			criterion.SetAttribute ("variable_instance", "1");
			
			if (testNegate.ToLower() == "true")
				criterion.SetAttribute ("negate", "true");
			
			criterion.SetAttribute ("result", Vocabulary.GetTestResult (result));
			
			resCritElement.AppendChild (criterion);
		}
	}
}
