// 
// OvalDocuments.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using System.Security.Cryptography;
using System.Text.RegularExpressions;
using System.Xml;
using System.Xml.Schema;
using System.Xml.XPath;
using System.Xml.Xsl;

namespace Sussen.Oval 
{
	public class OvalDocuments
	{
		DefinitionsDocument definitions;
		SystemCharacteristicsDocument systemCharacteristics;
		ResultsDocument results;
	
		public OvalDocuments ()
		{
			definitions = new DefinitionsDocument ();
			systemCharacteristics = new SystemCharacteristicsDocument ();
			results = new ResultsDocument (); 
		}
		
		public DefinitionsDocument Definitions
		{
			get { return definitions; }
			set { definitions = value; }
		}
		
		public SystemCharacteristicsDocument SystemCharacteristics
		{
			get { return systemCharacteristics; }
			set { systemCharacteristics = value; }
		}
		
		public ResultsDocument Results
		{
			get { return results; }
			set { results = value; }		
		}
	}
	
	public class Document
	{
		protected XmlDocument doc;
		protected bool validationErrors;
		
		public Document ()
		{
			doc = new XmlDocument ();
		}
		
		public void Load (string fileName)
		{
			if (!File.Exists (fileName))
				throw new FileNotFoundException (fileName);
							
			try {
				
				doc.Load (fileName);
							
			} catch (XmlException xe) {
			
				Log.Error (xe);			
				throw xe;
			} 		
		}
		
		public void LoadXml (string xmlData)
		{
			if (xmlData == null)
				throw new ArgumentNullException ("xmlData");
				
			try {
				doc.LoadXml (xmlData);
			} catch (XmlException xe) {
				Log.Error (xe);
				throw xe;
			}				
		}
		
		public void Save (string fileName)
		{
			try {
				
				doc.Save (fileName);
							
			} catch (Exception e) {
			
				Log.Error (e);			
				throw e;
			}		
		}		
		
		public bool CheckSchemaVersion ()
		{
			bool retVal = false;
		
			XmlNodeList elemList = doc.GetElementsByTagName("oval:schema_version");
			if (elemList.Count > 0) {
				
				string schema_version = elemList[0].InnerXml;
				if (schema_version.StartsWith ("5"))					
					retVal = true;
			}
			
			return retVal;
		}
		
		public static bool CheckFileHash (string fileName, string hashValue)
		{
			bool retVal = false;
			string fileHashString = null;
			byte[] fileHash = null;

			FileStream fs = File.OpenRead (fileName);

			MD5 md5sum = new MD5CryptoServiceProvider();

			fileHash = md5sum.ComputeHash (fs);			
			fileHashString = BitConverter.ToString(fileHash).Replace("-", "");

			fs.Close ();

			if (hashValue.ToLower() == fileHashString.ToLower())
				retVal = true;

			return retVal;
		}

		public XmlElement CreateElement (string elementName)
		{
			XmlElement xe = doc.CreateElement (elementName);
			return xe;
		}

		public XmlElement GetElement (string elementName)
		{
			XmlNodeList tmpList = doc.GetElementsByTagName (elementName);
			if (!(tmpList.Count > 0))
				return null;

			return (XmlElement) tmpList[0];
		}
		
		public XmlElement CopyElement (XmlElement origElement)
		{
			if (origElement == null)
				return null;
				
			XmlElement newElement = doc.CreateElement (origElement.Name, origElement.NamespaceURI);
					
			if (origElement.HasChildNodes)
				if (!(origElement.FirstChild.NodeType == XmlNodeType.Element))
					newElement.InnerText = origElement.InnerText;

			foreach (XmlAttribute a in origElement.Attributes) {
				// FIXME: Need some way to deal with schemas!
				if (a.Name.StartsWith ("xsi")) {
					continue;
				}
			
				newElement.SetAttribute (a.Name, a.Value);
			}

			foreach (XmlNode n in origElement.ChildNodes) {
			
				if (!(n.NodeType == XmlNodeType.Element))
					continue;
				
				XmlElement b = CopyElement ((XmlElement)n);	
				newElement.AppendChild (b);
			}
					
			return newElement;
		}
		
		public XmlNodeList GetElementsByName (string elementName)
		{
			return doc.GetElementsByTagName (elementName);
		}

		public override string ToString ()
		{
			return doc.InnerXml;
		}

		public bool Validate (string fileName)
		{
#if MONO_BUILD		
			// .NET 1.1 method
			try {
		
				XmlTextReader xr = new XmlTextReader (fileName);
				XmlValidatingReader vr = new XmlValidatingReader (xr);
				vr.ValidationEventHandler += new ValidationEventHandler (ValidationCallBack);
				vr.ValidationType = ValidationType.Schema;
		
				while (vr.Read()) ;
				
			} catch (XmlSchemaException xse) {
				Log.Error (xse.Message);
//				return false;
			}
#else
			// .NET 2.0 method
			try {
			
				XmlReaderSettings settings = new XmlReaderSettings();
				settings.ValidationType = ValidationType.Schema;
				settings.ValidationEventHandler += new ValidationEventHandler(ValidationCallBack);

				XmlReader reader = XmlReader.Create(configuration.XmlFileName, settings);
				while (reader.Read()) ;
				
			} catch (XmlException xe) {
				Log.Error (xe.Message);
				return false;
			}
#endif

			if (validationErrors)
				return false;

			return true;
		}
		
		void ValidationCallBack(object sender, ValidationEventArgs e)
        {       
			Log.Error ("Validation Error: {0}", e.Message);
			validationErrors = true;
		}	
	}

	public class EntityObject
	{
		string objName;
		string objValue;
		Variable variable;
		
		Operations operation;
		OvalDataTypes dataType;
		Checks checkType;
		
		bool nil;
	
		public EntityObject (string objectName, string objectValue, Operations op, OvalDataTypes dt, Checks check)
		{
			objName = objectName;
			objValue = objectValue;
			operation = op;
			dataType = dt;
			checkType = check;
		}
	
		public EntityObject (XmlElement entityElement)
		{
			if (entityElement == null)
				throw new ArgumentNullException ("entityElement");
				
			objName = entityElement.Name;
			objValue = entityElement.InnerText;
			
			operation = Vocabulary.GetOperation (entityElement.GetAttribute ("operation"));
			dataType = Vocabulary.GetDataType (entityElement.GetAttribute ("datatype"));
			checkType = Vocabulary.GetCheck (entityElement.GetAttribute ("var_check"));
			
			string n = entityElement.GetAttribute ("xsi:nil");
			if (n == null || n == "" || n == "false")
				nil = false;
			else
				nil = true;
			
			string varRef = entityElement.GetAttribute ("var_ref");
			if (varRef != "") {
				variable = Global.OvalVariables [varRef];
			}
		}
		
		public string Name
		{
			get { return objName; }
		}

		public bool Nil
		{
			get { return nil; }
		}

		public Operations Operation
		{
			get { return operation; }
		}

		public OvalDataTypes DataType
		{
			get { return dataType; }
		}
				
		public Checks Check
		{
			get { return checkType; }
		}

		public Variable VariableInstance
		{
			get { return variable; }
		}
				
		public string Value
		{
			get { return objValue; }
		}
	}

	public class EntityContainer : IEnumerable
	{
		ListDictionary etyDictionary;
	
		public EntityContainer ()
		{
			etyDictionary = new ListDictionary ();
		}

		public int Length
		{
			get { return etyDictionary.Count; }
		}

		public IEnumerator GetEnumerator ()
		{
			return etyDictionary.GetEnumerator ();
		}
		
		public EntityObject this [string name]
		{
			get { return (EntityObject) etyDictionary[name]; }
			set { etyDictionary[name] = value; }
		}		
	}
	
	public class ObjectElement
	{
		string name;
		string id;
		string version;
		string xmlns;
		
		EntityContainer entities;
		Hashtable behaviors;

		public ObjectElement (string objectName, string objectID, string objectVersion, string objectNS)
		{				
			name = objectName;
			id = objectID;
			version = objectVersion;
			xmlns = objectNS;

			CheckID (id);
			
			entities = new EntityContainer ();
			behaviors = new Hashtable ();
		}
	
		public ObjectElement (XmlElement objElement)
		{
			if (objElement == null)
				throw new ArgumentNullException ("objElement");

			name = objElement.Name;				
			id = objElement.GetAttribute ("id");
			version = objElement.GetAttribute ("version");
			xmlns = objElement.GetAttribute ("xmlns");
			
			CheckID (id);
			
			entities = new EntityContainer ();
			behaviors = new Hashtable ();
			
			foreach (XmlElement ee in objElement.ChildNodes)
				if (ee.NodeType == XmlNodeType.Element) {
					if (ee.Name == "behaviors")
						GetBehaviors (ee);
					else
						entities[ee.Name] = new EntityObject (ee);
				}
		}
		
		static void CheckID (string id)
		{
			Match m = Regex.Match (id, Vocabulary.OBJECT_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for object ID", "id");
			}
		}
		
		void GetBehaviors (XmlElement behaviorsElement)
		{
			if (behaviorsElement == null)
				throw new ArgumentNullException ("behaviorsElement");
				
			int numAttrs = behaviorsElement.Attributes.Count;
			for (int i = 0; i < numAttrs; i++) {
				XmlAttribute xa = behaviorsElement.Attributes[i]; 
				behaviors.Add (xa.Name, xa.Value);
			}
		}
		
		public Hashtable Behaviors
		{
			get { return behaviors; }
		}
		
		public EntityContainer Entities 
		{
			get { return entities; }
		}

		public string Name
		{
			get { return name; }
		}
		
		public string ID
		{
			get { return id; }
		}
		
		public string Version
		{
			get { return version; }
		}
		
		public string XmlNs
		{
			get { return xmlns; }
		}
	}
	
	public class ObjectContainer : IEnumerable
	{
		ListDictionary objDictionary;
	
		public ObjectContainer ()
		{
			objDictionary = new ListDictionary ();
		}

		public int Length
		{
			get { return objDictionary.Count; }
		}

		public IEnumerator GetEnumerator ()
		{
			return objDictionary.GetEnumerator ();
		}
		
		public ObjectElement this [string name]
		{
			get { return (ObjectElement) objDictionary[name]; }
			set { objDictionary[name] = value; }
		}		
	}

	public class StateContainer : IEnumerable
	{
		ListDictionary stateDictionary;
	
		public StateContainer ()
		{
			stateDictionary = new ListDictionary ();
		}

		public int Length
		{
			get { return stateDictionary.Count; }
		}

		public IEnumerator GetEnumerator ()
		{
			return stateDictionary.GetEnumerator ();
		}
		
		public StateElement this [string name]
		{
			get { return (StateElement) stateDictionary[name]; }
			set { stateDictionary[name] = value; }
		}		
	}
	
	public class StateElement
	{
		string name;
		string id;
		string version;
		string xmlns;

		EntityContainer entities;
		
		public StateElement (XmlElement steElement)
		{
			if (steElement == null)
				throw new ArgumentNullException ("steElement");

			name = steElement.Name;				
			id = steElement.GetAttribute ("id");
			version = steElement.GetAttribute ("version");
			xmlns = steElement.GetAttribute ("xmlns");

			Match m = Regex.Match (id, Vocabulary.STATE_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for state ID", "id");
			}
			
			entities = new EntityContainer ();
			foreach (XmlElement ee in steElement.ChildNodes)
				if (ee.NodeType == XmlNodeType.Element)
					entities[ee.Name] = new EntityObject (ee);
		}

		public string Name
		{
			get { return name; }
		}
			
		public string ID
		{
			get { return id; }
		}
		
		public string Version
		{
			get { return version; }
		}
		
		public string XmlNs
		{
			get { return xmlns; }
		}
		
		public EntityContainer Entities 
		{
			get { return entities; }
		}
	}

	public class TestContainer : IEnumerable
	{
		ListDictionary testDictionary;
	
		public TestContainer ()
		{
			testDictionary = new ListDictionary ();
		}

		public int Length
		{
			get { return testDictionary.Count; }
		}

		public IEnumerator GetEnumerator ()
		{
			return testDictionary.GetEnumerator ();
		}
		
		public TestElement this [string name]
		{
			get { return (TestElement) testDictionary[name]; }
			set { testDictionary[name] = value; }
		}		
	}
	
	public class TestElement
	{
		string name;
		string id;
		string version;
		string xmlns;
		string comment;		
		Checks check;		
		
		string objRef;
		string stateRef;
	
		public TestElement (XmlElement tstElement)
		{
			if (tstElement == null)
				throw new ArgumentNullException ("tstElement");

			name = tstElement.Name;				
			id = tstElement.GetAttribute ("id");
			version = tstElement.GetAttribute ("version");
			xmlns = tstElement.GetAttribute ("xmlns");
			comment = tstElement.GetAttribute ("comment"); 
			check = Vocabulary.GetCheck (tstElement.GetAttribute ("check"));

			Match m = Regex.Match (id, Vocabulary.TEST_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for test ID", "id");
			}
			
			foreach (XmlElement e in tstElement.ChildNodes)
				if (e.NodeType == XmlNodeType.Element) {				
					if (e.Name == "object")
						objRef = e.GetAttribute ("object_ref");
					else if (e.Name == "state")
						stateRef = e.GetAttribute ("state_ref");
				}
		}

		public string Name
		{
			get { return name; }
		}
			
		public string ID
		{
			get { return id; }
		}
		
		public string Version
		{
			get { return version; }
		}
		
		public string XmlNs
		{
			get { return xmlns; }
		}
		
		public string Comment
		{
			get { return comment; }
		}
		
		public Checks Check
		{
			get { return check; }
		}
		
		public string ObjectRef
		{
			get { return objRef; }
		}
		
		public string StateRef
		{
			get { return stateRef; }
		}
	}

	public class VariableContainer : IEnumerable
	{
		ListDictionary varDictionary;
	
		public VariableContainer ()
		{
			varDictionary = new ListDictionary ();
		}

		public int Length
		{
			get { return varDictionary.Count; }
		}

		public IEnumerator GetEnumerator ()
		{
			return varDictionary.GetEnumerator ();
		}
		
		public Variable this [string name]
		{
			get { return (Variable) varDictionary[name]; }
			set { varDictionary[name] = value; }
		}		
	}
	
	public class DefinitionsDocument : Document
	{
		ObjectContainer objects;
		TestContainer tests;
		StateContainer states;
		VariableContainer variables;
	
		public DefinitionsDocument () : base ()
		{
		}		

		public new void Load (string fileName)
		{
			base.Load (fileName);
			
			Collect ();
		}

		public new void LoadXml (string xmlData)
		{
			base.LoadXml (xmlData);
			
			Collect ();
		}

		void Collect ()
		{
			CollectVariables ();
			
			CollectTests ();
			CollectObjects ();
			CollectStates ();
		}

		void CollectTests ()
		{
			tests = new TestContainer ();

			XmlNodeList nl = doc.GetElementsByTagName ("tests");
			if (!(nl.Count > 0))
				return;
				
			XmlElement tstElement = (XmlElement) nl[0];
			foreach (XmlElement te in tstElement.ChildNodes) {
				if (te.NodeType == XmlNodeType.Element && te.Name.EndsWith ("_test")) {
					try {
						TestElement testElement = new TestElement (te);
						tests [testElement.ID] = testElement;
					} catch (ArgumentException ae) {
						Log.Debug (ae);
					}
				}
			}
		}

		void CollectObjects ()
		{
			objects  = new ObjectContainer ();
			
			XmlNodeList nl = doc.GetElementsByTagName ("objects");
			if (!(nl.Count > 0))
				return;
			
			XmlElement objElement = (XmlElement) nl[0];
			foreach (XmlElement oe in objElement.ChildNodes)
				if (oe.NodeType == XmlNodeType.Element && oe.Name.EndsWith ("_object")) {
					ObjectElement objectElement = new ObjectElement (oe);
					objects [objectElement.ID] = objectElement;
				}			
		}

		void CollectStates ()
		{
			states  = new StateContainer ();
			
			XmlNodeList nl = doc.GetElementsByTagName ("states");
			if (!(nl.Count > 0))
				return;
				
			XmlElement statesElement = (XmlElement) nl[0];
			foreach (XmlElement oe in statesElement.ChildNodes)
				if (oe.NodeType == XmlNodeType.Element && oe.Name.EndsWith ("_state")) {
					StateElement stateElement = new StateElement (oe);
					states [stateElement.ID] = stateElement;
				}
				
			Global.SystemObjects = states;
		}

		void CollectVariables ()
		{
			variables = new VariableContainer ();
		
			XmlNodeList nl = doc.GetElementsByTagName ("variables");
			if (!(nl.Count > 0))
				return;
				
			XmlElement varElement = (XmlElement) nl[0];
			foreach (XmlElement ve in varElement.ChildNodes)
				if (ve.NodeType == XmlNodeType.Element && ve.Name.EndsWith ("_variable")) {
					Variable ovalVariable = VariableFactory.Create (ve);
					variables [ovalVariable.ID] = ovalVariable;
				}
				
			Global.OvalVariables = variables;
		}

		public ObjectContainer Objects
		{
			get { return objects; }
		}
		
		public TestContainer Tests
		{
			get { return tests; }
		}
		
		public StateContainer States
		{
			get { return states; }
		}
		
		public VariableContainer Variables
		{
			get { return variables; }
		}
	}

	public struct SystemInterfaceData 
	{
		public string Name;
		public string IpAddress;
		public string MacAddress;
	}

	public abstract class SystemInfo
	{
		public SystemInfo ()
		{
		}
		
		public abstract string OsName { get; }
		public abstract string OsVersion { get; }
		public abstract string OsArchitecture { get; }
		public abstract string PrimaryHostName { get; }
		public abstract SystemInterfaceData[] Interfaces { get; }
	}

	public class CollectedObjectsContainer : IEnumerable
	{
		ListDictionary coDictionary;
	
		public CollectedObjectsContainer ()
		{
			coDictionary = new ListDictionary ();
		}

		public int Length
		{
			get { return coDictionary.Count; }
		}

		public IEnumerator GetEnumerator ()
		{
			return coDictionary.GetEnumerator ();
		}
		
		public CollectedObjectElement this [string name]
		{
			get { return (CollectedObjectElement) coDictionary[name]; }
			set { coDictionary[name] = value; }
		}		
	}
	
	public class CollectedObjectElement
	{
		string id;
		string version;
		Flags flag;
		string itemRef;
	
		public CollectedObjectElement (XmlElement element)
		{
			if (element == null)
				throw new ArgumentNullException ("element");

			id = element.GetAttribute ("id");
			version = element.GetAttribute ("version");
			flag = Vocabulary.GetFlag (element.GetAttribute ("flag"));
			
			foreach (XmlElement e in element.ChildNodes)
				if (e.NodeType == XmlNodeType.Element)		
					if (e.Name == "reference")
						itemRef = e.GetAttribute ("item_ref");
		}
	
		public string ID
		{
			get { return id; }
		}
		
		public string Version
		{
			get { return version; }
		}
		
		public Flags Flag
		{
			get { return flag; }
		}
		
		public string ItemRef
		{
			get { return itemRef; }
		}
	}

	public class SystemDataContainer : IEnumerable
	{
		ListDictionary sdDictionary;
	
		public SystemDataContainer ()
		{
			sdDictionary = new ListDictionary ();
		}

		public int Length
		{
			get { return sdDictionary.Count; }
		}

		public IEnumerator GetEnumerator ()
		{
			return sdDictionary.GetEnumerator ();
		}
		
		public CollectedItem this [string name]
		{
			get { return (CollectedItem) sdDictionary[name]; }
			set { sdDictionary[name] = value; }
		}		
	}

	public class SystemCharacteristicsDocument : Document
	{
		XmlElement system_characteristics;
		XmlElement system_info;
		XmlElement collected_objects;
		XmlElement system_data;
		
		CollectedObjectsContainer collectedObjects;
		SystemDataContainer systemData;
		
		public SystemCharacteristicsDocument () : base ()
		{			 
		}
		
		public void Create ()
		{
			XmlNode xmlnode = doc.CreateNode (XmlNodeType.XmlDeclaration,"","");
			doc.AppendChild (xmlnode);
	
			system_characteristics = doc.CreateElement("oval_system_characteristics");
			system_characteristics.SetAttribute ("xmlns", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			system_characteristics.SetAttribute ("xmlns:oval", "http://oval.mitre.org/XMLSchema/oval-common-5");
			system_characteristics.SetAttribute ("xmlns:oval-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			system_characteristics.SetAttribute ("xmlns:oval-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			system_characteristics.SetAttribute ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
//			system_characteristics.SetAttribute ("xsi:schemaLocation", "http://oval.mitre.org/XMLSchema/oval-definitions-5 oval-definitions-schema.xsd");		
			doc.AppendChild (system_characteristics);
			
			WriteGenerator ();

			system_info = doc.CreateElement ("system_info");
			system_characteristics.AppendChild (system_info);
			
			collected_objects = doc.CreateElement ("collected_objects");
			system_characteristics.AppendChild (collected_objects);
			
			system_data = doc.CreateElement ("system_data");
			system_characteristics.AppendChild (system_data);		
		}
		
		public void AddNamespace (string nsName, string nsUri)
		{
			if (nsName == null)
				throw new ArgumentNullException (nsName);

			if (nsUri == null)
				throw new ArgumentNullException (nsUri);				
				
			system_characteristics.SetAttribute (nsName, nsUri);
		}
		
		public void AddSystemData (XmlElement itemElement)
		{
			if (itemElement == null)
				throw new ArgumentNullException ("itemElement");
				
			system_data.AppendChild (itemElement);
		}
		
		public void WriteCollectedObject (string objectID, string version, Flags flag, string message, MessageLevels level)
		{
			XmlElement objectElement = doc.CreateElement ("object");
			objectElement.SetAttribute ("id", objectID);
			objectElement.SetAttribute ("version", version);
			objectElement.SetAttribute ("flag", Vocabulary.GetFlag (flag));
			collected_objects.AppendChild (objectElement);
			
			XmlElement xe = doc.CreateElement ("message");
			xe.SetAttribute ("level", Vocabulary.GetMessageLevel (level));			
			xe.InnerText = message;
				
			objectElement.AppendChild (xe);			
		}
		
		public void WriteCollectedObject (string objectID, string version, Flags flag, int itemReference)
		{
			XmlElement objectElement = doc.CreateElement ("object");			
			objectElement.SetAttribute ("id", objectID);
			objectElement.SetAttribute ("version", version);
			objectElement.SetAttribute ("flag", Vocabulary.GetFlag (flag));
			collected_objects.AppendChild (objectElement);
			
			XmlElement refElement = doc.CreateElement ("reference");
			refElement.SetAttribute ("item_ref", itemReference.ToString ());
			objectElement.AppendChild (refElement);			
		}
		
		void GetCollectedObjects ()
		{
			collectedObjects  = new CollectedObjectsContainer ();
			
			XmlNodeList nl = doc.GetElementsByTagName ("collected_objects");
			if (!(nl.Count > 0))
				return;
				
			XmlElement element = (XmlElement) nl[0];
			foreach (XmlElement oe in element.ChildNodes)
				if (oe.NodeType == XmlNodeType.Element) {
					CollectedObjectElement coElement = new CollectedObjectElement (oe); 
					collectedObjects [coElement.ID] = coElement;
				}		
		}
		
		void GetSystemData ()
		{
			systemData = new SystemDataContainer ();

			XmlNodeList nl = doc.GetElementsByTagName ("system_data");
			if (!(nl.Count > 0))
				return;
				
			XmlElement sdElement = (XmlElement) nl[0];
			foreach (XmlElement sdc in sdElement.ChildNodes) {
			
				int id = int.Parse (sdc.GetAttribute ("id"));
				CollectedItem ci = new CollectedItem (sdc.Name, id, sdc.GetAttribute ("xmlns")); 
			
				foreach (XmlElement oe in sdc.ChildNodes)
					if (oe.NodeType == XmlNodeType.Element) {

						ItemEntity ie = new ItemEntity (
							oe.Name, 
							oe.InnerText,
							Vocabulary.GetDataType (oe.GetAttribute ("datatype")),
							Vocabulary.GetStatusFlag (oe.GetAttribute ("status")),
							false);
						
						ci.Flag = Flags.Complete;
						ci.AddItem (ie);					
						systemData [ci.ID.ToString()] = ci;
					}
			}
		}
		
		void WriteGenerator ()
		{
			XmlElement generator;		
			generator = doc.CreateElement ("generator");
			
			XmlElement product_name = doc.CreateElement ("oval:product_name", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_name.InnerText = Defines.PACKAGE;
			generator.AppendChild (product_name);
			
			XmlElement product_version = doc.CreateElement ("oval:product_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_version.InnerText = Defines.VERSION;
			generator.AppendChild (product_version);

			XmlElement schema_version = doc.CreateElement ("oval:schema_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			schema_version.InnerText = "5.0";
			generator.AppendChild (schema_version);
			
			XmlElement timestamp = doc.CreateElement ("oval:timestamp", "http://oval.mitre.org/XMLSchema/oval-common-5");
			timestamp.InnerText = String.Format ("{0:yyyy-MM-ddTHH:mm:ss}", DateTime.Now);
			generator.AppendChild (timestamp);
						
			system_characteristics.AppendChild (generator);		
		}

		public void WriteSystemInfo (SystemInfo platformSystemInfo)
		{
			XmlElement os_name = doc.CreateElement ("os_name");
			os_name.InnerText = platformSystemInfo.OsName;
			system_info.AppendChild (os_name);
			
			XmlElement os_version = doc.CreateElement ("os_version");
			os_version.InnerText = platformSystemInfo.OsVersion;
			system_info.AppendChild (os_version);

			XmlElement arch = doc.CreateElement ("architecture");
			arch.InnerText = platformSystemInfo.OsArchitecture;
			system_info.AppendChild (arch);
				
			XmlElement primary_host_name = doc.CreateElement ("primary_host_name");
			primary_host_name.InnerText = platformSystemInfo.PrimaryHostName;
			system_info.AppendChild (primary_host_name);
			
			XmlElement interfaces = doc.CreateElement ("interfaces");

			foreach (SystemInterfaceData ifd in platformSystemInfo.Interfaces) {
			
				XmlElement xe = doc.CreateElement ("interface");
				
				XmlElement name = doc.CreateElement ("interface_name");
				name.InnerText = ifd.Name;
				xe.AppendChild (name);

				XmlElement ip = doc.CreateElement ("ip_address");
				ip.InnerText = ifd.IpAddress;
				xe.AppendChild (ip);
				
				XmlElement mac = doc.CreateElement ("mac_address");
				mac.InnerText = ifd.MacAddress;
				xe.AppendChild (mac);
				
				interfaces.AppendChild (xe);
			}

			system_info.AppendChild (interfaces);
		}
		
		public CollectedObjectsContainer CollectedObjects
		{
			get {
				if (collectedObjects == null)
					GetCollectedObjects();
					
				return collectedObjects;
			}
		}

		public SystemDataContainer SystemData
		{
			get {
				if (systemData == null)
					GetSystemData();
					
				return systemData;
			}
		}		
	}
	
	public class ResultsDocument : Document
	{
		XmlElement oval_results;
		XmlElement oval_definitions;
		XmlElement results;
		XmlElement system;
		XmlElement definitions;
		XmlElement tests;
		XmlElement oval_system_characteristics;
		
		public ResultsDocument () : base ()
		{
			doc.AppendChild (doc.CreateNode (XmlNodeType.XmlDeclaration,"",""));
			
			WriteHeader ();
			WriteGenerator ();
			
			oval_results.AppendChild (doc.CreateElement ("directives"));
			
			oval_definitions = doc.CreateElement ("oval_definitions", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_results.AppendChild (oval_definitions);
			
			results = doc.CreateElement ("results");
			oval_results.AppendChild (results);
			
			system = doc.CreateElement ("system");			
			definitions = doc.CreateElement ("definitions");
			system.AppendChild (definitions);
			
			tests = doc.CreateElement ("tests");
			system.AppendChild (tests);

			oval_system_characteristics = doc.CreateElement ("oval_system_characteristics", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			system.AppendChild (oval_system_characteristics);			
			
			results.AppendChild (system);			
		}

		public void AddTest (XmlElement testElement)
		{
			if (testElement == null)
				throw new ArgumentNullException ("testElement");
				
			tests.AppendChild (testElement);
		}

		public void CopyDefinitions (DefinitionsDocument definitionDocument)
		{
			XmlElement genElement =  definitionDocument.GetElement ("generator");
			oval_definitions.AppendChild (this.CopyElement (genElement));
			
			XmlElement defElement = definitionDocument.GetElement ("definitions");
			oval_definitions.AppendChild (this.CopyElement (defElement));

			XmlElement tstElement = definitionDocument.GetElement ("tests");
			oval_definitions.AppendChild (this.CopyElement (tstElement));

			XmlElement objElement = definitionDocument.GetElement ("objects");
			oval_definitions.AppendChild (this.CopyElement (objElement));

			XmlElement steElement = definitionDocument.GetElement ("states");
			oval_definitions.AppendChild (this.CopyElement (steElement));

			XmlElement varElement = definitionDocument.GetElement ("variables");
			if (varElement != null)
				oval_definitions.AppendChild (this.CopyElement (varElement));
		}

		public void CopySystemCharacteristics (SystemCharacteristicsDocument scDocument)
		{
			XmlElement genElement = scDocument.GetElement ("generator");
			oval_system_characteristics.AppendChild (this.CopyElement (genElement));
			
			XmlElement siElement = scDocument.GetElement ("system_info");
			oval_system_characteristics.AppendChild (this.CopyElement (siElement));

			XmlElement cobElement = scDocument.GetElement ("collected_objects");
			oval_system_characteristics.AppendChild (this.CopyElement (cobElement));

			XmlElement sdElement = scDocument.GetElement ("system_data");
			oval_system_characteristics.AppendChild (this.CopyElement (sdElement));		
		}

		public void Save (Config currentConfiguration)
		{
			if (currentConfiguration.ResultsFileFormat == ResultOutputTypes.XML) {

				doc.Save (currentConfiguration.ResultsFile);

			} else {

				XslTransform xslt = new XslTransform();
				xslt.Load (currentConfiguration.GetXmlResource ("results_to_html.xsl"));
			
				string tmp = Path.GetTempFileName ();
				File.Delete (tmp);			

				doc.Save (tmp);	
				XPathDocument resPathDoc = new XPathDocument(tmp);
			
				XmlWriter writer = new XmlTextWriter (currentConfiguration.ResultsFile, null); 
				xslt.Transform (resPathDoc, null, writer, null);
				writer.Close ();
			
				File.Delete (tmp);
			}		
		}

		void WriteHeader ()
		{
			oval_results = doc.CreateElement ("oval_results");
			oval_results.SetAttribute ("xmlns", "http://oval.mitre.org/XMLSchema/oval-results-5");
			oval_results.SetAttribute ("xmlns:oval", "http://oval.mitre.org/XMLSchema/oval-common-5");
			oval_results.SetAttribute ("xmlns:oval-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_results.SetAttribute ("xmlns:oval-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			oval_results.SetAttribute ("xmlns:oval-res", "http://oval.mitre.org/XMLSchema/oval-results-5");
			oval_results.SetAttribute ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			
			doc.AppendChild (oval_results);
		}
		
		void WriteGenerator ()
		{
			XmlElement generator;		
			generator = doc.CreateElement ("generator");
			
			XmlElement product_name = doc.CreateElement ("oval:product_name", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_name.InnerText = Defines.PACKAGE;
			generator.AppendChild (product_name);
			
			XmlElement product_version = doc.CreateElement ("oval:product_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_version.InnerText = Defines.VERSION;
			generator.AppendChild (product_version);

			XmlElement schema_version = doc.CreateElement ("oval:schema_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			schema_version.InnerText = "5.0";
			generator.AppendChild (schema_version);
			
			XmlElement timestamp = doc.CreateElement ("oval:timestamp", "http://oval.mitre.org/XMLSchema/oval-common-5");
			timestamp.InnerText = String.Format ("{0:yyyy-MM-ddTHH:mm:ss}", DateTime.Now);
			generator.AppendChild (timestamp);
						
			oval_results.AppendChild (generator);		
		}

	}	
}