//
// Test.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval
{
	public enum DefinitionResults {
		True,
		False,
		Unknown,
		Error,
		NotEvaluated,
		NotApplicable
	};

	public enum TestOperations {
		Equals,
		NotEqual,
		GreaterThan,
		LessThan,
		GreaterThanOrEqual,
		LessThanOrEqual,
		BitwiseAnd,
		BitwiseOr,
		PatternMatch
	};

	public enum TestOperators {
		And,
		Or,
		Xor
	};

	public struct TestMetaData
	{
		public string ID;
		public string Comment;
		public string Check;
		public string TestType;
		
		public Hashtable TestObjects;
		public Hashtable TestStates;
	};

	public class ItemEntity
	{
		string objName;
		string objValue;		
		OvalDataTypes dataType;	
		StatusFlags status;
		bool isTestObject;
	
		public ItemEntity (string itemName, string itemValue, OvalDataTypes itemDatatype, StatusFlags itemStatus, bool testObject)
		{
			objName = itemName;
			objValue = itemValue;
			dataType = itemDatatype;
			status = itemStatus;
			isTestObject = testObject;
		}
		
		public void Write (SystemCharacteristicsDocument scDoc, XmlElement parentItem)
		{
			XmlElement item = scDoc.CreateElement (objName);
			item.InnerText = objValue;
			
			string dtName = Vocabulary.GetDataType (dataType);
			if (dtName != "string")
				item.SetAttribute ("datatype", dtName);
				
			string stName = Vocabulary.GetStatusFlag (status);
			if (stName != "exists")
				item.SetAttribute ("status", stName);
				
			parentItem.AppendChild (item);
		}
		
		public string Name
		{
			get { return objName; }
		}
		
		public string Value
		{
			get { return objValue; }
		}
		
		public bool IsTestObject
		{
			get { return isTestObject; }
		}
		
		public StatusFlags Status
		{
			get { return status; }
		}
	}

	public class CollectedItem
	{
		string name;
		int id;
		string xmlns;
		string message;
		MessageLevels level;
		Flags flag;
		
		ArrayList items;

		public CollectedItem (string itemName, int itemID, string xmlNamespace)
		{
			name = itemName;
			id = itemID;
			xmlns = xmlNamespace;
			
			items = new ArrayList ();
		}
	
		public CollectedItem (string itemName, int itemID, string xmlNamespace, Flags itemFlag)
		{
			name = itemName;
			id = itemID;
			xmlns = xmlNamespace;
			flag = itemFlag;
			
			items = new ArrayList ();
		}
		
		public void AddItem (ItemEntity item)
		{
			items.Add (item);
		}
		
		public override string ToString ()
		{
			StringBuilder sb = new StringBuilder ();
			sb.AppendFormat ("Item: {0}\n", name);
			sb.AppendFormat ("ID: {0}\n", id);
			
			if (message != null)
				sb.AppendFormat ("Message: {0}\n", message);
			
			foreach (ItemEntity ie in items)
				sb.AppendFormat ("{0}:{1}\tStatus: {2}\n", ie.Name, ie.Value, Vocabulary.GetStatusFlag (ie.Status));

			return sb.ToString();			
		}
		
		public void Write (SystemCharacteristicsDocument scDoc)
		{
			XmlElement xe = scDoc.CreateElement (name);
			xe.SetAttribute ("id", id.ToString());
			xe.SetAttribute ("xmlns", xmlns);
			
			foreach (ItemEntity ie in items) {
				ie.Write (scDoc, xe);
			}
			
			scDoc.AddSystemData (xe);
		}
		
		public Flags Flag
		{
			get { return flag; }
			set { flag = value; }
		}
		
		public int ID
		{
			get { return id; }
			set { id = value; }
		}
		
		public ItemEntity[] Items
		{
			get { return (ItemEntity[]) items.ToArray (typeof (ItemEntity)); }
		}
		
		public string Message
		{
			get { return message; }
			set { message = value; }
		}
		
		public MessageLevels MessageLevel
		{
			get { return level; }
			set { level = value; }
		}
	}

	public abstract class Test
	{
		public Test ()
		{
		}

		protected static bool CompareItems (string lhs, string rhs, Operations operation)
		{
			bool retVal = false;
			
			switch (operation) {

			case Operations.Equals:
				if (lhs == rhs)
					retVal = true;
				break;
					
			case Operations.NotEqual:
				if (lhs != rhs)
					retVal = true;
				break;
				
			case Operations.PatternMatch:
				// FIXME: OVAL uses POSIX format for regex; not sure .NET supports that
				Regex r = new Regex (lhs, RegexOptions.None);
				Match m = r.Match (rhs);
				if (m.Success)
					retVal = true;			
				break;
				
			default:
				throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (operation));
			}
			
			return retVal;
		}

		public abstract CollectedItem[] Run (TestElement testElement, ObjectElement objectElement);
	}

	public class TestException : Exception
	{
		string id;
		
		public TestException () : base ()
		{
		}

		public TestException (string testID, string message) : base (message)
		{
		}
		
		public string ID
		{
			get { return id; }
		}
	}
}
