// 
// TestAnalyzer.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval
{
	public class TestAnalyzer : Analyzer
	{
		OvalDocuments docs;
	
		public TestAnalyzer ()
		{
		}

		public TestResult Run (TestElement testElement, OvalDocuments ovalDocuments)
		{
			if (testElement == null)
				throw new ArgumentNullException ("testElement");

			if (ovalDocuments == null)
				throw new ArgumentNullException ("ovalDocuments");
				
			docs = ovalDocuments;
			TestResult testResult = TestResult.Unknown;
			
			ArrayList objectResults = new ArrayList ();
			StateElement stateElement = null;
						
			if (testElement.StateRef != null)
				stateElement = docs.Definitions.States [testElement.StateRef];
						
			CollectedObjectElement coElement = docs.SystemCharacteristics.CollectedObjects [testElement.ObjectRef];
	
			if (coElement == null)
				return testResult;
				
			if (coElement.ItemRef == null)
				return testResult;
				
			CollectedItem ci = docs.SystemCharacteristics.SystemData [coElement.ItemRef];
			
			if (stateElement != null) {
				// FIXME: handle more than one reference_item
				TestResult res = CompareCollectedDataToState (stateElement, ci);
				objectResults.Add (res);				
			} else {
		
				TestResult res = TestResult.Unknown;

				foreach (ItemEntity ie in ci.Items)
					if (ie.Status == StatusFlags.Exists)
						res = TestResult.True;
					else
						res = TestResult.Error;

				objectResults.Add (res);			
			}

			testResult = AnalyzeResultsByCheck (objectResults, Vocabulary.GetCheck (testElement.Check));

			Log.Debug ("Analyzed test: [{0}] version: [{1}] check: [{2}] result: [{3}]",
				testElement.ID, testElement.Version, Vocabulary.GetCheck (testElement.Check), 
				Vocabulary.GetTestResult (testResult));

			WriteTest (testElement.ID, testElement.Version, 
				Vocabulary.GetCheck (testElement.Check), 
				Vocabulary.GetTestResult (testResult), 
				coElement.ItemRef);
										
			return testResult;
		}
				
		void WriteTest (string testId, string testVersion, string testCheck, string testResult, string itemId)
		{
			XmlElement test = docs.Results.CreateElement ("test"); 
			test.SetAttribute ("test_id", testId);
			test.SetAttribute ("version", testVersion);
			test.SetAttribute ("variable_instance", "1");
			test.SetAttribute ("check", testCheck);
			test.SetAttribute ("result", testResult);
			
			if (itemId != null) {
				XmlElement tested_item = docs.Results.CreateElement ("tested_item");
				tested_item.SetAttribute ("item_id", itemId);
				tested_item.SetAttribute ("result", testResult);
			
				test.AppendChild (tested_item);
			}
			
			docs.Results.AddTest (test);
		}

		TestResult CompareCollectedDataToState (StateElement stateElement, CollectedItem systemData)
		{
			TestResult res = TestResult.Unknown;
		
			foreach (DictionaryEntry de in stateElement.Entities) {
				EntityObject eo = (EntityObject) de.Value;			
				foreach (ItemEntity ie in systemData.Items)
					if (ie.Name == eo.Name)
						res = CompareData (Vocabulary.GetDataType (eo.DataType), Vocabulary.GetOperation (eo.Operation), eo.Value, ie.Value); 
			}
				
			return res;
		}

		TestResult CompareData (string stateDataType, string stateOperation, string stateValue, string dataValue)
		{
			TestResult result = TestResult.Unknown;
			
			if (stateOperation == "")
				stateOperation = "equals";
					
			if (stateDataType == "")
				stateDataType = "string";

			switch (stateDataType) {
			case "binary":
//				result = CompareBinaryItems (stateValue, dataValue, stateOperation);
				break;
					
			case "boolean":
				result = CompareBoolItems (stateValue, dataValue, stateOperation);
				break;

			case "evr_string":
//				result = CompareEvrStringItems (stateValue, dataValue, stateOperation);
				break;
					
			case "float":
				result = CompareFloatItems (stateValue, dataValue, stateOperation);			
				break;

			case "ios_version":
//				result = CompareIosVersionItems (stateValue, dataValue, stateOperation);
				break;
					
			case "int":
				result = CompareIntItems (stateValue, dataValue, stateOperation);
				break;
					
			case "string":
				result = CompareStringItems (stateValue, dataValue, stateOperation);			
				break;

			case "version":
				// FIXME: compare versions not just the strings
				result = CompareStringItems (stateValue, dataValue, stateOperation);			
				break;
					
			default:
				throw new ArgumentOutOfRangeException ("Invalid data type");
			}
			
			return result;
		}

		TestResult CompareBoolItems (string b1, string b2, string itemOperator)
		{
			TestResult retVal = TestResult.Unknown;
			bool a, b;
			
			if (b1.ToLower() == "true" || b1 == "1")
				a =  true;
			else
				a = false;
				
			if (b2.ToLower() == "true" || b2 == "1")
				b = true;
			else
				b = false;
						
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case "equals":
				if (a == b)
					retVal = TestResult.True;
				break;
				
			case "not equal":
				if (a != b)
					retVal = TestResult.True;
				break;
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareFloatItems (string f1, string f2, string itemOperator)
		{
			TestResult retVal = TestResult.Unknown;
			double a, b;
			
			try {
				a = double.Parse (f1);
				b = double.Parse (f2);				
			} catch {
				retVal = TestResult.Error;
				return retVal;
			}
			
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case "equals":
				if (a == b)
					retVal = TestResult.True;
				break;
				
			case "not equal":
				if (a != b)
					retVal = TestResult.True;
				break;
				
			case "less than":
				if (a < b)
					retVal = TestResult.True;			
				break;
				
			case "greater than":
				if (a > b)
					retVal = TestResult.True;			
				break;
				
			case "less than or equal":
				if (a <= b)
					retVal = TestResult.True;			
				break;
				
			case "greater than or equal":
				if (a >= b)
					retVal = TestResult.True;
				break;
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareIntItems (string i1, string i2, string itemOperator)
		{
			TestResult retVal = TestResult.Unknown;
			int a, b;
			
			try {
				a = int.Parse (i1);
				b = int.Parse (i2);				
			} catch {
				retVal = TestResult.Error;
				return retVal;
			}
			
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case "equals":
				if (a == b)
					retVal = TestResult.True;
				break;
				
			case "not equal":
				if (a != b)
					retVal = TestResult.True;
				break;
				
			case "less than":
				if (a < b)
					retVal = TestResult.True;			
				break;
				
			case "greater than":
				if (a > b)
					retVal = TestResult.True;			
				break;
				
			case "less than or equal":
				if (a <= b)
					retVal = TestResult.True;			
				break;
				
			case "greater than or equal":
				if (a >= b)
					retVal = TestResult.True;			
				break;
			
			case "bitwise and":
				if ((a & b) == a)
					retVal = TestResult.True;			
				break;
				
			case "bitwise or":
				if ((a | b) == a)
					retVal = TestResult.True;			
				break;
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareStringItems (string s1, string s2, string itemOperator)
		{
			TestResult retVal = TestResult.Unknown;

			if (s1 == null || s2 == null || itemOperator == "")
				return retVal;
			
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case "equals":
				if (s1 == s2)
					retVal = TestResult.True;
				break;
				
			case "not equal":
				if (s1 != s2)
					retVal = TestResult.True;
				break;
				
			case "less than":
				if ((string.Compare (s1, s2) < 0))
					retVal = TestResult.True;			
				break;
				
			case "greater than":
				if ((string.Compare (s1, s2) > 0))
					retVal = TestResult.True;			
				break;
				
			case "less than or equal":
				if ((string.Compare (s1, s2) <= 0))
					retVal = TestResult.True;			
				break;
				
			case "greater than or equal":
				if ((string.Compare (s1, s2) >= 0))
					retVal = TestResult.True;			
				break;

			case "pattern match":
				Regex r = new Regex (s1, RegexOptions.IgnoreCase);
				Match m = r.Match (s2);
				if (m.Success)
					retVal = TestResult.True;							
				break;				
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}
	}
}
