//
// Variable.cs - NUnit Test Cases for Sussen.Oval
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Independent;

namespace SussenTests.Oval
{
	[TestFixture]
	public class VariableTests
	{
		OvalDocuments docs;

		[SetUp]
		public void SetUp()
		{
			docs = new OvalDocuments ();
			docs.Definitions.Load ("variable.xml");
		}

		[TearDown]
		public void TearDown ()
		{
			docs = null;
		}

		[Test]
		public void ConstantSingleValueTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");
			
			ConstantVariable cv = (ConstantVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:1"];
			if (cv == null)
				Assert.Fail ("excepted variable 'oval:com.mmgsecurity:var:1' not found");

			Assert.AreEqual ("foo", cv.Value);
		}

		[Test]
		public void ConstantMultipleStringValueTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");
			
			ConstantVariable cv = (ConstantVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:2"];
			if (cv == null)
				Assert.Fail ("excepted variable 'oval:com.mmgsecurity:var:2' not found");

			Assert.AreEqual (6, cv.ValueArray.Length);
			Assert.AreEqual ("foo", cv.ValueArray[0]);
			Assert.AreEqual ("bar", cv.ValueArray[1]);
			Assert.AreEqual ("who", cv.ValueArray[2]);
			Assert.AreEqual ("what", cv.ValueArray[3]);
			Assert.AreEqual ("where", cv.ValueArray[4]);
			Assert.AreEqual ("when", cv.ValueArray[5]);
		}

		[Test]
		public void IndependentVariableTest ()
		{	
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			VariableTest vt = new VariableTest ();
			ObjectElement oe = new ObjectElement ("variable_object", "oval:com.mmgsecurity:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("var_ref", "oval:com.mmgsecurity:var:1", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			CollectedItem[] testResults;
			testResults = vt.Run (null, oe);

			foreach (CollectedItem c in testResults) {
				ItemEntity ie = c.Items[0];
				if (ie.Name == "value")
					Assert.AreEqual ("foo", ie.Value);
			}
		}

		[Test]
		public void EntityObjectVariableTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			ObjectElement oe = docs.Definitions.Objects["oval:com.mmgsecurity:obj:1"];
			Assert.IsNotNull (oe);

			EntityObject path = oe.Entities["path"];
			Assert.IsNotNull (path);
			Assert.IsNotNull (path.VariableInstance);
			Assert.AreEqual ("foo", path.VariableInstance.Value);
		}

		[Test]
		public void LocalVariableSimpleTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			LocalVariable lv = (LocalVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:5"];
			Assert.IsNotNull (lv);
			Assert.AreEqual ("rusty", lv.Value);
		}

		[Test]
		public void LocalVariableConcatTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			LocalVariable lv = (LocalVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:3"];
			Assert.IsNotNull (lv);

			Assert.AreEqual (2, lv.ValueArray.Length);
			Assert.AreEqual ("foobar", lv.ValueArray[0]);
			Assert.AreEqual ("somethingelse", lv.ValueArray[1]);
		}

		[Test]
		public void LocalVariableSplitTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			LocalVariable lv = (LocalVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:4"];
			Assert.IsNotNull (lv);

			Assert.AreEqual (3, lv.ValueArray.Length);
			Assert.AreEqual ("12", lv.ValueArray[0]);
			Assert.AreEqual ("00", lv.ValueArray[1]);
			Assert.AreEqual ("00", lv.ValueArray[2]);
		}

		[Test]
		public void LocalVariableSubstringTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			LocalVariable lv = (LocalVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:6"];
			Assert.IsNotNull (lv);
			Assert.AreEqual ("needle", lv.Value);
		}

		[Test]
		public void LocalVariableEscapeRegexTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			LocalVariable lv = (LocalVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:7"];
			Assert.IsNotNull (lv);
			Assert.AreEqual (@"some\*thing\+different", lv.Value);
		}

		[Test]
		public void LocalVariableBeginTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			LocalVariable lv = (LocalVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:8"];
			Assert.IsNotNull (lv);
			Assert.AreEqual ("xxxastring", lv.Value);
		}

		[Test]
		public void LocalVariableEndTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			LocalVariable lv = (LocalVariable) docs.Definitions.Variables ["oval:com.mmgsecurity:var:9"];
			Assert.IsNotNull (lv);
			Assert.AreEqual ("astringxxx", lv.Value);
		}
	}
}
